const updaterProdutosModel = require('../../model/updaterProdutosModel');
const updaterProdVersaoModel = require('../../model/updaterProdVersaoModel');
const updaterHistoricoModel = require('../../model/updaterHistoricoModel');
const updaterServiceManagementFili = require('../../model/updaterServiceManagementModel');
const updaterModel = require('../../model/updaterModel');
const socketService = require('../../service/SocketService');

const route = (router) => {
  router.get('/updaterProdutos', (async (req, res) => {
    updaterProdutosModel.getUpdaterProdutos(true).then((products) => {
      let { rows } = products;
      const {
        search,
        search_in
      } = req.query || {};
      if(search && typeof search_in === 'string') {
        rows = rows.filter(row => {
          return (row[search_in] || '').toString().toLowerCase().includes(search);
        });
      }
      const response = { 
        data: rows,
        pagination: {
          limit: 999,
          total: rows.length
        }
      };
      res.send(response);
    }).catch((error) => {
      res.send(error.message || error);
    });
  }));

  router.get('/updaterProdutosPerCasher', (async (req, res) => {
    const productId = req.query.PRODUCT_ID;
    updaterProdutosModel.getUpdaterProdutosPerCasherFilial(productId).then((products) => {
        const response = { 
          data: products.rows,
          pagination: {
            limit: 999,
            total: products.rows.length
          }
        };
        res.send(response);
            
    }).catch((error) => {
      res.send(error.message || error);
    });
  }));

  router.get('/updaterProdutosFilial', (async (req, res) => {
    const productId = req.query.PRODUCT_ID;
    if(!productId) {
      res.send([]);
    } else {
      updaterProdutosModel.getUpdaterProdutosWithUnits(productId).then(async (products) => {
        let updaterSockets = await socketService.getUpdaterSocket().getUpdaters();
        const connectedFiliais = [];
        updaterSockets.forEach((socket) => {
          if(!socket.CDCAIXA) {
            connectedFiliais.push(socket.CDFILIAL);
          }
        });
        const unitsWithWarning = socketService.getUpdaterSocket().getUnitsWithWarning(true);
        products.rows = products.rows.map((product) => {
          product.CONNECTION_STATUS = connectedFiliais.includes(product.CDFILIAL);
          product.WARNING = unitsWithWarning.includes(product.CDFILIAL);
          return product;
        });
        const response = { 
          data: products.rows,
          pagination: {
            limit: 999,
            total: products.rows.length          
          }
        };
        res.send(response);
      }).catch((error) => {
        res.send(error.message || error);
      });
    }
  }));

  router.post('/updaterProductMultiInstall', (async (req, res) => {
    const { PRODUCT_ID, UNITS } = req.body;
    if(!PRODUCT_ID) {
      res.send([]);
      return;
    }
    try {
      const productVersions = await updaterProdutosModel.getUpdaterProductsWithVersion(PRODUCT_ID);
      const updaterSockets = await socketService.getUpdaterSocket().getUpdaters();
      const connectedUnits = [];
      updaterSockets.forEach((socket) => {
        if(!socket.CDCAIXA) {
          connectedUnits.push(socket.CDFILIAL);
        }
      });
      const onlySelectedUnits = ({ CDFILIAL }) => UNITS.includes(CDFILIAL);
      const fillConnectionStatus = row => row.CONNECTION_STATUS = connectedUnits.includes(row.CDFILIAL);
      productVersions.rows = productVersions.rows.filter(onlySelectedUnits);
      productVersions.rows.forEach(fillConnectionStatus);
      const response = { 
        data: productVersions.rows,
        pagination: {
          limit: 999,
          total: productVersions.rows.length
        }
      };
      res.send(response);
    } catch (error) {
      res.status(500).send(error.message || error);
    }
  }));

  router.post('/getEventsInProgressByProduct', async (req, res) => {
    const { PRODUCT_ID, UNITS } = req.body;
    if(!PRODUCT_ID) {
      return res.status(400).send('PRODUCT_NOT_INFORMED');
    }
    try {
      const EVENTS_IN_PROGRESS = await updaterModel.getEventsInProgressByProduct({
        PRODUCT_ID,
        CDFILIAL: `_${UNITS.join('_')}_`
      });
      res.send({
        EVENTS: EVENTS_IN_PROGRESS.rows
      });
    } catch ( error ) {
      res.status(500).send(error.message || error);
    }
  });

  router.get('/updaterProdutosFilialPerCasher', (async (req, res) => {
    updaterProdutosModel.getUpdaterProdutosWithUnitsPerCasher(req.query).then(async (products) => {        
      let socketUpdaters = await socketService.getUpdaterSocket().getUpdaters();
      const connectedFiliais = socketUpdaters.reduce((acc, socket) => {
        acc[socket.CDFILIAL] = acc[socket.CDFILIAL] || new Set();
        if(socket.CDCAIXA) {
          acc[socket.CDFILIAL].add(socket.CDCAIXA);
        }
        return acc;
      }, {});
      const unitsWithWarning = socketService.getUpdaterSocket().getUnitsWithWarning();
      products.rows = products.rows.map((product) => {
        const curUnit = connectedFiliais[product.CDFILIAL] || new Set();
        const cashersWithWarning = unitsWithWarning[product.CDFILIAL] || [];
        product.CONNECTION_STATUS = curUnit.has(product.CDCAIXA);
        product.WARNING = cashersWithWarning.includes(product.CDCAIXA);
        return product;
      });
      const response = { 
        data: products.rows,
        pagination: {
          limit: 999,
          total: products.rows.length          
        }
      };
      res.send(response);
    }).catch((error) => {
      res.send(error.message || error);
    });
}));

  router.get('/updaterProdutosVersao', (async (req, res) => {
    const productId = req.query.CDPRODUCTID;     
    const { search } = req.query || {};
    const { USE_VERSION_FILTER } = process.env;
    if(!productId) {
      res.send([]);
    } else {
      updaterProdVersaoModel.getUpdaterProdVersaoByProduct(productId,  USE_VERSION_FILTER).then((products) => {             
        let { rows } = products;     
        if(search) {          
          rows = rows.filter(row => {           
            return (row.NRVERSAO || '').includes(search);
          });
        }  
        
        const response = {
          data: rows,
          pagination: {
            limit: 999,
            total: products.rows.length
          }
        };
        res.send(response);
      
      }).catch((error) => {
        res.send(error.message || error);
      });
    }
  }));

  router.get('/updaterProdutosVersaoConfig', (async (req, res) => {
    updaterProdVersaoModel.getUpdaterProdVersaoConfig().then((products) => {
      const response = { 
        data: products.rows,
        pagination: {
          limit: 999,
          total: products.rows.length
        }
      };
      res.send(response);
    }).catch((error) => {
      res.send(error.message || error);
    });
  }));

  router.get('/updaterProdutoHistorico', (async (req, res) => {
    const cdProductId = req.query.CDPRODUCTID;
    const cdFilial = req.query.CDFILIAL;
    updaterHistoricoModel.getUpdaterProdutoHistorico(cdProductId, cdFilial).then((historico) => {
      const response = {
        data: historico.rows,
        pagination: {
          limit: 999,
          total: historico.rows.length
        }
      };
      res.send(response);
    }).catch((error) => {
      res.send(error.message || error);
    });
  }));

  router.get('/updaterProdutoHistoricoPorCaixa', (async (req, res) => {
    const cdProductId = req.query.CDPRODUCTID;
    const cdCaixa = req.query.CDCAIXA;
    const cdFilial = req.query.CDFILIAL;
    updaterHistoricoModel.getUpdaterProdutoHistoricoPorCaixa(cdProductId, cdFilial, cdCaixa).then((historico) => { 
      const response = { 
        data: historico.rows,
        pagination: {
          limit: 999,
          total: historico.rows.length
        }
      };
      res.send(response);
    }).catch((error) => {
      res.send(error.message || error);
    });
  }));

  router.put('/updateUpdaterProdVersaoAprov', (async (req, res) => {
    try {
      const result = await updaterProdVersaoModel.updateUpdaterProdVersaoAprov({
        ID: req.body.ID,
        IDAPROVVER: req.body.IDAPROVVER,
      });
      
      result.connection.commit();
      res.send('Sucesso!');
    } catch (error) {
      res.send(error.message || error);
    }
  }));

  router.get('/updaterServiceManagementFili', (async (req, res) => {  
  const { search } = req.query || {}; 
    updaterServiceManagementFili.filterServiceManagementFili().then(({ rows }) => {     

      if(search) {          
        rows = rows.filter(row => {           
          return (row.FILIAL || '').includes(search);
        });
      }  

      const response = {
        data: rows,
        pagination: {
          limit: 999,
          total: rows.length
        }
      };
      res.send(response);
    }).catch((error) => {
      res.send(error.message || error);
    });
  }));
  
  router.get('/updaterServiceManagement', (async (req, res) => {
    const cdFilial = req.query.CDFILIAL;

    updaterServiceManagementFili.filterServiceManagementMonitoring(cdFilial).then(({ rows }) => { 
     let serviceManagement = []; 
      rows.map((element) => {
        const service = element.MONITORAMENTO.split(';');
        service.forEach((item) => {
          serviceManagement.push({
            CDFILIAL: element.CDFILIAL,
            MONITORAMENTO: item
          })
        })
      })      
     
      const response = {
        data: serviceManagement,
        pagination: {         
          limit: 999,
          total: serviceManagement.length
        }
      };
      res.send(response);
    }).catch((error) => {
      res.send(error.message || error);
    });
  }));

  router.get('/minimumSupportedVersion', async (req, res) => {
    const { MIN_UPDATER_VERSION_SUPPORTED } = process.env;
    res.send({ minimumSupportedVersionUpdater: MIN_UPDATER_VERSION_SUPPORTED });
  })
}

module.exports = route;