const JWT = require('../functions/JsonWebToken');

class Auth {
  isAuth(req, res, next){
    const token = req.headers.authorization || '';
    if (token === '' && !this.checkRouteAuthorized(req.path)) {
      res.status(401).send('Not Authorized');
      return;
    }

    if (token !== '') {
      const verification = JWT.verify(token);
      
      if ('valid' in verification && verification.valid === false) {
        res.status(401).send('Unauthorized');
        return;
      }
    }

    next();
    return;
  }

  checkRouteAuthorized(route) {
    const authorizedRoutes = this.getAuthorizedRoutes().concat(this.getScreenRoutes());
    const isPublicPath = this.isPublicPath(route);
    return isPublicPath || this.isUpdaterRoute(route) || authorizedRoutes.some((authorized) => route === authorized || route === `${authorized}/`);
  }

  isUpdaterRoute(route) {
    return route.includes('/updater/');
  }

  isPublicPath(route) {
    return route.includes('/img/') ||
           route.includes('/metadata/') ||
           route.includes('/precache') ||
           route.includes('/assets/');
  }

  getScreenRoutes() {
    return [
      '/monitoring',
      '/monitoring-cloud',
      '/updates',
      '/parametrizacoes',
      '/version-config',
      '/up-version',
      '/update-per-casher'
    ];
  }

  getAuthorizedRoutes() {
    return [
      '/',
      '/index.html',
      '/manifest.json',
      '/service-worker.js',
      '/robots.txt',
      '/favicon.ico',
      '/login'
    ];
  }
}

module.exports = new Auth()