const OracleService = require('../service/OracleService');

class parametrizacoesModel {
  constructor() {
    this.GET_ALL = `
      SELECT * FROM PORTAL ORDER BY ORDEM, NOME
    `;

    this.GET_ASS_CONFIG = `
      SELECT ID, NOME, DESCRICAO, TIPO, ORDEM, ATIVO, COMANDO, INTERVALO, COMPLEMENTOTIPO
        FROM ASSISTANT_CONFIG 
    ORDER BY ORDEM, NOME
    `;

    this.GET_ASS_CONFIG_ACTIVE = `
      SELECT ID, NOME, DESCRICAO, TIPO, ORDEM, ATIVO, COMANDO, INTERVALO, COMPLEMENTOTIPO
        FROM ASSISTANT_CONFIG 
       WHERE ATIVO = 'S'
    ORDER BY ORDEM, NOME
    `;

    this.UPDATE_ASS_CONFIG = `
      UPDATE ASSISTANT_CONFIG
         SET NOME      = :NOME,
             DESCRICAO = :DESCRICAO,
             TIPO      = :TIPO,
             ORDEM     = :ORDEM,
             ATIVO     = :ATIVO,
             COMANDO   = :COMANDO,
             INTERVALO = :INTERVALO,
             COMPLEMENTOTIPO = :COMPLEMENTOTIPO
       WHERE ID = :ID
    `;

    this.INSERT_ASS_CONFIG = `
      INSERT INTO ASSISTANT_CONFIG (NOME, DESCRICAO, TIPO, ORDEM, COMANDO, INTERVALO, COMPLEMENTOTIPO, ATIVO)
                            VALUES (:NOME, :DESCRICAO, :TIPO, :ORDEM, :COMANDO, :INTERVALO, :COMPLEMENTOTIPO, :ATIVO)
    `;

    this.DELETE_ASS_CONFIG = `
      DELETE 
        FROM ASSISTANT_CONFIG
       WHERE ID IN (:ID)
    `;

    this.GET_FILIAL = `
        SELECT CDFILIAL, NMFILIAL
          FROM FILIAL
          WHERE IDATIVO = 'S'
      ORDER BY CDFILIAL
    `;

    this.GET_MONITORING_PARAMS = `
    SELECT
      ID,
      NOME,
      TIPO,
      COMANDO,
      INTERVALO,
      COMPLEMENTOTIPO
    FROM 
      ASSISTANT_CONFIG
    WHERE ATIVO = 'S'
    `;

    this.GET_MONITORING_APACHE_CLOUD = `
    SELECT
      ID,
      NOME,
      DESCRICAO,
      TIPO,
      COMANDO,
      INTERVALO,
      COMPLEMENTOTIPO
    FROM 
      ASSISTANT_CONFIG
    WHERE ATIVO = 'S' AND TIPO = 'C'
    `;

    this.GET_UPDATER_SERVICO_FILI = `
    SELECT
      CDFILIAL,
      MONITORAMENTO
    FROM
      UPDATERSERVICOFILI
    WHERE
      MONITORAMENTO IS NOT NULL
    `
  }

  get() {
    return OracleService.connect().then(() => {
      return OracleService.execute(this.GET_ALL);
    });
  }

  getAssConfig(activeOnly = false) {
    return OracleService.connect().then(() => {
      const sql = activeOnly ? this.GET_ASS_CONFIG_ACTIVE : this.GET_ASS_CONFIG;
      return OracleService.fetchAssoc(sql);
    });
  }

  async getMonitoringParams(apacheCloudOnly = false){
    const sql = apacheCloudOnly ? this.GET_MONITORING_APACHE_CLOUD : this.GET_MONITORING_PARAMS;
    return OracleService.fetchAssoc(sql);
  }

  async getUpdaterServicoFili(){
    return OracleService.connect().then(() => {
      return OracleService.fetchAssoc(this.GET_UPDATER_SERVICO_FILI);
    });
  }

  async updateMonitoringConfig(params) {
    return OracleService.connect().then(() => {
      return OracleService.execute(this.UPDATE_ASS_CONFIG, params);
    });
  }

  async insertMonitoringConfig(params) {
    return OracleService.connect().then(() => {
      return OracleService.execute(this.INSERT_ASS_CONFIG, params, [{name: 'ID', type: 'number'}]);
    });
  }

  async deleteMonitoringConfig(params) {
    return OracleService.connect().then(() => {
      return OracleService.execute(this.DELETE_ASS_CONFIG.replace(':ID', params.ID));
    });
  }

  getFilial() {
    return OracleService.connect().then(() => {
      return OracleService.fetchAssoc(this.GET_FILIAL);
    });
  }
}

module.exports = new parametrizacoesModel();
