const OracleService = require('../service/OracleService');

class updaterModel {
  constructor() {
    /**
     * UPDATER
     */ 
    this.GET_UPDATER = `
      SELECT
      F.CDFILIAL,
      F.NMFILIAL,
      UPS.*,
      EN.SGESTADO
      FROM
        UPDATERSISTEMAS UPS INNER JOIN
        FILIAL F ON (F.CDFILIAL = UPS.CDFILIAL) INNER JOIN
        ENDEFILI EN ON (F.CDFILIAL = EN.CDFILIAL)
      WHERE EN.IDTPENDEFILI = 'P'
    `;

    /**
     * UPDATERSISTEMAS
     */
    this.GET_UPDATERSISTEMAS = `
     SELECT * FROM UPDATERSISTEMAS
      WHERE NOME = :NOME
   `;

    this.GET_FULL_UPDATERSISTEMAS = `
     SELECT IDUPDATERSISTEMAS, DEVOPSID, SISTEMASSTATUS, CDFILIAL FROM UPDATERSISTEMAS
   `;

    /**
     * UPDATERVERSAO
     */
    this.GET_UPDATERVERSAO = `
     SELECT * FROM UPDATERVERSAO
      WHERE IDUPDATERSISTEMAS = :IDUPDATERSISTEMAS AND UPVERSTATUS = 0
   `;

    this.SET_UPDATERVERSAO = `
      INSERT INTO UPDATERVERSAO
        (IDUPDATERSISTEMAS, CDOPERADOR, UPVERDATA, UPVERSTATUS)
        VALUES (:IDUPDATERSISTEMAS, :CDOPERADOR, :UPVERDATA, :UPVERSTATUS)
   `;

   this.SET_DOWNGRADEVERSAO = `
      INSERT INTO UPDATERVERSAO
        (IDUPDATERSISTEMAS, CDOPERADOR, UPVERDATA, UPVERSTATUS)
        VALUES (:IDUPDATERSISTEMAS, :CDOPERADOR, :UPVERDATA, :UPVERSTATUS)
   `;

    /**
     * Histórico
     */
    this.GET_UPDATER_HISTORICO = `
      SELECT OP.NMOPERADOR, OP.CDOPERADOR, UPR.*, UPS.*, UPV.*
        FROM FILIAL F INNER JOIN
          UPDATERSISTEMAS UPS ON (F.CDFILIAL = UPS.CDFILIAL) INNER JOIN
          UPDATERVERSAO UPV ON (UPS.IDUPDATERSISTEMAS = UPV.IDUPDATERSISTEMAS) INNER JOIN
          OPERADOR OP ON (UPV.CDOPERADOR = OP.CDOPERADOR)
    `;
    /**
     * Histórico
     */
    this.GET_UPDATER_V_ATUAL = `
      SELECT
          F.CDFILIAL,
          F.NMFILIAL,
          OP.NMOPERADOR,
          OP.CDOPERADOR,
          UPS.*,
          UPV.*
        FROM FILIAL F INNER JOIN
          UPDATERSISTEMAS UPS ON (F.CDFILIAL = UPS.CDFILIAL) INNER JOIN
          UPDATERVERSAO UPV ON (UPS.IDUPDATERSISTEMAS = UPV.IDUPDATERSISTEMAS) INNER JOIN
          OPERADOR OP ON (UPV.CDOPERADOR = OP.CDOPERADOR)

        WHERE
          UPS.IDUPDATERSISTEMAS = :IDUPDATERSISTEMAS AND
          (UPV.UPVERSTATUS = 0 OR UPV.UPVERSTATUS = 3) AND
          ROWNUM <= :MAX_ROW
        ORDER BY UPV.IDUPDATERVERSAO DESC
   `;

    this.GET_UPDATER_V_ALL = `
      SELECT
          F.CDFILIAL,
          F.NMFILIAL,
          OP.NMOPERADOR, OP.CDOPERADOR, UPS.*, UPV.*
        FROM FILIAL F INNER JOIN
          UPDATERSISTEMAS UPS ON (F.CDFILIAL = UPS.CDFILIAL) INNER JOIN
          UPDATERVERSAO UPV ON (UPS.IDUPDATERSISTEMAS = UPV.IDUPDATERSISTEMAS) INNER JOIN
          OPERADOR OP ON (UPV.CDOPERADOR = OP.CDOPERADOR)

        WHERE
          UPS.IDUPDATERSISTEMAS = :IDUPDATERSISTEMAS AND
          UPS.CDFILIAL = :CDFILIAL AND
          ROWNUM <= :MAX_ROW
        ORDER BY UPV.IDUPDATERVERSAO DESC
   `;

   /**
    * Retorna os sistemas a serem atualizados para cilial cdFilial
    */
   this.GET_UPDATER_ESPERA_ALL = `
   SELECT UPS.APLICACAO, UPV.IDUPDATERVERSAO, UPS.IDUPDATERSISTEMAS, UPS.DIRETORIO
      FROM
        FILIAL F INNER JOIN
        UPDATERSISTEMAS UPS ON (F.CDFILIAL = UPS.CDFILIAL) INNER JOIN
        UPDATERVERSAO UPV ON (UPS.IDUPDATERSISTEMAS = UPV.IDUPDATERSISTEMAS) INNER JOIN
        OPERADOR OP ON (UPV.CDOPERADOR = OP.CDOPERADOR)

      WHERE
        F.CDFILIAL = :cdfilial AND
        UPV.UPVERSTATUS = 0 AND
        UPS.DIRETORIO IS NOT NULL
      GROUP BY UPS.APLICACAO, UPV.IDUPDATERVERSAO, UPS.IDUPDATERSISTEMAS, UPS.DIRETORIO
   `;

   /**
    * Retorna os sistemas a terem downgrade para cilial cdFilial
    */
   this.GET_DOWNGRADE_ESPERA_ALL = `
   SELECT UPS.APLICACAO, UPV.IDUPDATERVERSAO, UPS.IDUPDATERSISTEMAS, UPS.DIRETORIO
      FROM
        FILIAL F INNER JOIN
        UPDATERSISTEMAS UPS ON (F.CDFILIAL = UPS.CDFILIAL) INNER JOIN
        UPDATERVERSAO UPV ON (UPS.IDUPDATERSISTEMAS = UPV.IDUPDATERSISTEMAS) INNER JOIN
        OPERADOR OP ON (UPV.CDOPERADOR = OP.CDOPERADOR)

      WHERE
        F.CDFILIAL = :cdfilial AND
        UPV.UPVERSTATUS = 3 AND
        UPS.DIRETORIO IS NOT NULL
      GROUP BY UPS.APLICACAO, UPV.IDUPDATERVERSAO, UPS.IDUPDATERSISTEMAS, UPS.DIRETORIO
   `;

   this.UPDATE_UPDATER = `
      UPDATE UPDATERVERSAO
        SET UPVERSTATUS = :UPVERSTATUS
      WHERE IDUPDATERVERSAO = :IDUPDATERVERSAO
   `;

   this.UPDATE_UPDATERSISTEMA_ATUAL = `
      UPDATE UPDATERSISTEMAS
        SET UPSISVERATUAL = :UPSISVERATUAL
      WHERE IDUPDATERSISTEMAS = :IDUPDATERSISTEMAS
   `;

   this.SET_UPDATERSISTEMAS = `
    INSERT INTO UPDATERSISTEMAS
                (CDFILIAL,   DEVOPSID,  SISTEMASSTATUS,  APLICACAO,  DIRETORIO)
         VALUES (:CDFILIAL, :DEVOPSID, :SISTEMASSTATUS, :APLICACAO, :DIRETORIO)
  `;

   this.SET_UPDATERSISTEMAS_CANCELED = `
      UPDATE UPDATERSISTEMAS
         SET SISTEMASSTATUS = 2
       WHERE DEVOPSID = :DEVOPSID
         AND CDFILIAL = :CDFILIAL
  `;

   this.GET_PRODUCT_PENDING_EVENTS = `
      SELECT 
        * 
      FROM 
        UPDATEREVENTOS 
      WHERE 
        CDFILIAL = :P_CDFILIAL AND 
        CDPRODUCTID = :P_CDPRODUCTID AND 
        IDSTEVENTO IN ('INSTALACAO_PENDENTE', 'REMOCAO_PENDENTE')
  `;

   this.GET_PRODUCT_UPDATER_EVENT_BY_EVENT_ID = `
      SELECT 
        * 
      FROM 
        UPDATEREVENTOS 
      WHERE 
        CDFILIAL = :P_CDFILIAL AND 
        CDPRODUCTID = :P_CDPRODUCTID AND 
        IDSTEVENTO = :P_IDSTEVENTO
  `;

   this.GET_PRODUCT_IN_PROGRESS_EVENTS = `
      SELECT 
        * 
      FROM 
        UPDATEREVENTOS 
      WHERE 
        CDFILIAL = :P_CDFILIAL AND 
        CDPRODUCTID = :P_CDPRODUCTID AND 
        IDSTEVENTO IN ('INSTALACAO_EM_CURSO', 'REMOCAO_EM_CURSO')
  `;

   this.GET_EVENTS_BY_UNIT = `
      SELECT 
        * 
      FROM 
        UPDATEREVENTOS 
      WHERE 
        CDFILIAL = :P_CDFILIAL AND 
        IDSTEVENTO IN ('INSTALACAO_PENDENTE', 'REMOCAO_PENDENTE', 'INSTALACAO_EM_CURSO', 'REMOCAO_EM_CURSO')
  `;

   this.GET_IN_PROGRESS_EVENTS = `
      SELECT 
        * 
      FROM 
        UPDATEREVENTOS 
      WHERE 
        CDFILIAL = :P_CDFILIAL AND 
        IDSTEVENTO IN ('INSTALACAO_EM_CURSO', 'REMOCAO_EM_CURSO')
  `;

  this.GET_IN_PROGRESS_EVENTS_PRODUCT = `
    SELECT
      CDEVENTO
    FROM
      UPDATEREVENTOS EV
    WHERE
      EV.CDPRODUCTID = :PRODUCT_ID
      AND INSTR(:CDFILIAL, '_' || EV.CDFILIAL || '_') <> 0
      AND EV.IDSTEVENTO IN (
          'REMOCAO_PENDENTE',
          'REMOCAO_EM_CURSO',
          'INSTALACAO_EM_CURSO'
      )
  `;

   this.GET_PENDING_EVENTS = `
      SELECT 
        UE.CDFILIAL,
        UE.CDEVENTO,
        UE.CDPRODUCTID,
        UE.IDSTEVENTO,
        UE.NRVERSAO,
        UE.IDCRIARVERSAO,
        UE.DTINCLUSAO,
        UPV.DSDIRETORIO,
        UPV.IDDOWNLOADVER
      FROM 
        UPDATEREVENTOS UE
        LEFT JOIN UPDATERPRODVERSAO UPV ON (UE.CDPRODUCTID = UPV.UPDATERPRODID AND UE.NRVERSAO = UPV.NRVERSAO)
      WHERE 
        IDSTEVENTO IN ('INSTALACAO_PENDENTE', 'REMOCAO_PENDENTE')
  `;

   this.GET_UPDATER_PENDING_EVENTS = `
      SELECT 
        UE.CDFILIAL,
        UPH.CDOPERADOR,
        UE.CDEVENTO,
        UE.CDPRODUCTID,
        UE.CDPRODFILIID,
        UE.IDSTEVENTO,
        UE.NRVERSAOATUAL,
        UE.NRVERSAOALVO,
        UE.IDCRIARVERSAO,
        UE.DTINCLUSAO,
        UPV.DSDIRETORIO,
        UPV.IDDOWNLOADVER,
        UPC.CONFIG
      FROM 
        UPDATEREVENTOS UE
        LEFT JOIN UPDATERPRODVERSAO UPV ON (UE.CDPRODUCTID = UPV.UPDATERPRODID AND UE.NRVERSAOALVO = UPV.NRVERSAO)
        LEFT JOIN UPDATERPRODCONFIG UPC ON (UE.CDPRODUCTID = UPC.CDPRODUCTID AND UE.NRVERSAOALVO = UPC.NRVERSAO)
        LEFT JOIN UPDATERHISTORICO UPH ON (UE.CDEVENTO = UPH.CDEVENTO)
      WHERE 
        UE.CDFILIAL = :P_CDFILIAL AND 
        UE.IDSTEVENTO IN ('INSTALACAO_PENDENTE', 'INSTALACAO_EM_CURSO')
      UNION
      SELECT 
        UE.CDFILIAL,
        UPH.CDOPERADOR,
        UE.CDEVENTO,
        UE.CDPRODUCTID,
        UE.CDPRODFILIID,
        UE.IDSTEVENTO,
        UE.NRVERSAOATUAL,
        UE.NRVERSAOALVO,
        UE.IDCRIARVERSAO,
        UE.DTINCLUSAO,
        UPV.DSDIRETORIO,
        UPV.IDDOWNLOADVER,
        '' AS CONFIG
      FROM 
        UPDATEREVENTOS UE
        LEFT JOIN UPDATERPRODVERSAO UPV ON (UE.CDPRODUCTID = UPV.UPDATERPRODID AND UE.NRVERSAOATUAL = UPV.NRVERSAO)
        LEFT JOIN UPDATERHISTORICO UPH ON (UE.CDEVENTO = UPH.CDEVENTO)
      WHERE 
        UE.CDFILIAL = :P_CDFILIAL AND 
        UE.IDSTEVENTO IN ('REMOCAO_PENDENTE', 'REMOCAO_EM_CURSO')
  `;

      this.GET_PENDING_EVENTS_BY_CASHIER = `
        SELECT 
          UE.CDFILIAL,
          UE.CDCAIXA,
          UPH.CDOPERADOR,
          UE.CDEVENTO,
          UE.CDPRODUCTID,
          UE.CDPRODFILIID,
          UE.IDSTEVENTO,
          UE.NRVERSAOATUAL,
          UE.NRVERSAOALVO,
          UE.IDCRIARVERSAO,
          UE.DTINCLUSAO,
          UPV.DSDIRETORIO,
          UPV.IDDOWNLOADVER,
          UPC.CONFIG       
        FROM 
          UPDATEREVENTOSCX UE
          LEFT JOIN UPDATERPRODVERSAO UPV ON (UE.CDPRODUCTID = UPV.UPDATERPRODID AND UE.NRVERSAOALVO = UPV.NRVERSAO)
          LEFT JOIN UPDATERPRODCONFIG UPC ON (UE.CDPRODUCTID = UPC.CDPRODUCTID AND UE.NRVERSAOALVO = UPC.NRVERSAO)
          LEFT JOIN UPDATERHISTORICOCX UPH ON (UE.CDEVENTO = UPH.CDEVENTO)          
        WHERE 
          UE.CDFILIAL = :P_CDFILIAL AND
          UE.CDCAIXA = :P_CDCAIXA AND
          UE.IDSTEVENTO IN ('INSTALACAO_PENDENTE', 'INSTALACAO_EM_CURSO')
        UNION
        SELECT 
          UE.CDFILIAL,
          UE.CDCAIXA,
          UPH.CDOPERADOR,
          UE.CDEVENTO,
          UE.CDPRODUCTID,
          UE.CDPRODFILIID,
          UE.IDSTEVENTO,
          UE.NRVERSAOATUAL,
          UE.NRVERSAOALVO,
          UE.IDCRIARVERSAO,
          UE.DTINCLUSAO,
          UPV.DSDIRETORIO,
          UPV.IDDOWNLOADVER,
          '' AS CONFIG
        FROM 
          UPDATEREVENTOSCX UE
          LEFT JOIN UPDATERPRODVERSAO UPV ON (UE.CDPRODUCTID = UPV.UPDATERPRODID AND UE.NRVERSAOATUAL = UPV.NRVERSAO)
          LEFT JOIN UPDATERHISTORICOCX UPH ON (UE.CDEVENTO = UPH.CDEVENTO)
        WHERE 
          UE.CDFILIAL = :P_CDFILIAL AND 
          UE.CDCAIXA = :P_CDCAIXA AND
          UE.IDSTEVENTO IN ('REMOCAO_PENDENTE', 'REMOCAO_EM_CURSO')
`;

  this.GET_UPDATER_HISTORY_BY_EVENT_ID = `
    SELECT
      *
    FROM
      UPDATERHISTORICO
    WHERE
      CDEVENTO = :P_CDEVENTO AND
      IDSTEVENTO = :P_IDSTEVENTO
  `;

  this.GET_CONFIG_INI_HISTORY_BY_UNIT = `
    SELECT 
      U.ID, 
      U.CDOPERADOR, 
      O.NMOPERADOR, 
      TO_CHAR(U.DTULTATU, 'DD/MM/YYYY HH24:MI:SS') AS DTULTATU
    FROM 
      UPDATERCONFIGHIST U
      JOIN OPERADOR O ON (U.CDOPERADOR = O.CDOPERADOR)
    WHERE 
      U.UPDATERCONFIGID = :P_ID
    ORDER BY 
      ID DESC
  `;

   this.APPEND_UPDATER_HISTORY = `
      INSERT INTO UPDATERHISTORICO (
        CDFILIAL,
        CDHISTORICO,
        CDEVENTO,
        CDOPERADOR,
        CDUUID,
        CDPRODUCTID,
        NMPRODUCT,
        IDSTEVENTO,
        NRVERSAOATUAL,
        NRVERSAOALVO,
        IDCRIARVERSAO,
        DTINCLUSAO,
        DTULTATU
      ) VALUES (
        :P_CDFILIAL,
        :P_CDHISTORICO,
        :P_CDEVENTO,
        :P_CDOPERADOR,
        :P_CDUUID,
        :P_CDPRODUCTID,
        :P_NMPRODUCT,
        :P_IDSTEVENTO,
        :P_NRVERSAOATUAL,
        :P_NRVERSAOALVO,
        :P_IDCRIARVERSAO,
        SYSDATE,
        SYSDATE
      )
    `;

    this.APPEND_CONFIG_INI_HISTORY = `
      INSERT INTO UPDATERCONFIGHIST (
        UPDATERCONFIGID, 
        CDOPERADOR, 
        CONFIG,
        DTINCLUSAO,
        DTULTATU
      ) VALUES (
        :P_UPDATERCONFIGID, 
        :P_CDOPERADOR, 
        :P_CONFIG,
        SYSDATE,
        SYSDATE
      )
    `;

   this.INSERT_PENDING_EVENT = `
      INSERT INTO UPDATEREVENTOS (
        CDFILIAL,
        CDEVENTO,
        CDUUID,
        CDPRODUCTID,
        CDPRODFILIID,
        IDSTEVENTO,
        NRVERSAOATUAL,
        NRVERSAOALVO,
        IDCRIARVERSAO,
        DTINCLUSAO,
        DTULTATU
      ) VALUES (
        :P_CDFILIAL,
        :P_CDEVENTO,
        :P_CDUUID,
        :P_CDPRODUCTID,
        :P_CDPRODFILIID,
        :P_IDSTEVENTO,
        :P_NRVERSAOATUAL,
        :P_NRVERSAOALVO,
        :P_IDCRIARVERSAO,
        SYSDATE,
        SYSDATE
      )
    `;

   this.UPDATE_EVENT_IN_PROGRESS = `
      UPDATE UPDATEREVENTOS
      SET
        IDSTEVENTO = :P_IDSTEVENTO,
        DTULTATU = SYSDATE
      WHERE
        CDEVENTO = :P_CDEVENTO
    `;

   this.UPDATE_CONFIG_INI_BY_UNIT = `
      UPDATE 
        UPDATERCONFIG
      SET 
        CONFIG = :P_CONFIG,
        DTULTATU = SYSDATE
      WHERE 
        CDFILIAL = :P_CDFILIAL
    `;

   this.CANCEL_UNIT_EVENTS = `
      UPDATE 
        UPDATEREVENTOS
      SET
        IDSTEVENTO = 'CANCELADO',
        DTULTATU = SYSDATE
      WHERE
        CDFILIAL = :P_CDFILIAL
    `;

    this.GET_VERSION = `
      SELECT 
        ID
      FROM
        UPDATERPRODFILI
      WHERE
        CDFILIAL = :P_CDFILIAL AND
        UPDATERPRODID = :P_UPDATERPRODID AND
        NRVERSAOATUAL = :P_NRVERSAOATUAL
    `; 
    
    /* consulta, atualização e inserção no config.ini */
    this.GET_CONFIG_INI = `
      SELECT 
        ID,
        CONFIG 
      FROM 
        UPDATERCONFIG
      WHERE 
        CDFILIAL = :P_CDFILIAL
    `;

    this.INSERT_CONFIG_INI = `
      INSERT INTO UPDATERCONFIG (
        CDFILIAL, 
        CONFIG
      ) VALUES (
        :P_CDFILIAL, 
        :P_CONFIG
      )
    `;

    this.GET_DEFAULT_OPERATOR = `
      SELECT 
        CDOPERADOR,
        CDVENDEDOR 
      FROM 
        PARAVEND
      WHERE 
        CDFILIAL = :P_CDFILIAL
    `;
    
    this.GET_DEFAULT_OPERATOR_FROM_SALESMAN = `
      SELECT 
        CDOPERADOR 
      FROM 
        VENDEDOR
      WHERE 
        CDVENDEDOR = :P_CDVENDEDOR
    `;

    this.GET_CONFIG_INI_BY_UNIT = `
      SELECT 
        ID,
        CONFIG,
        DTULTATU
      FROM 
        UPDATERCONFIG
      WHERE 
        CDFILIAL = :P_CDFILIAL
    `;

    this.GET_SERVICE_MONITOR_CONFIGURATION_BY_UNIT = `
        SELECT
          MONITORAMENTO 
        FROM 
          UPDATERSERVICOFILI 
        WHERE 
          CDFILIAL = :P_CDFILIAL
    `;

    this.UPDATE_SERVICE_MONITOR_CONFIGURATION_BY_UNIT = `
        UPDATE
          UPDATERSERVICOFILI  
        SET
          MONITORAMENTO = :P_MONITORAMENTO 
        WHERE 
          CDFILIAL = :P_CDFILIAL
    `;
    
    this.GET_UPDATER_PROD_CONFIG  = `
    SELECT       
      CONFIG      
    FROM 
      UPDATERPRODCONFIG
    WHERE 
      CDPRODUCTID = :CDPRODUCTID AND
      NRVERSAO = :NRVERSAO
  `;

  this.GET_UPDATER_PROD_CONFIG_LAST_UPDATE  = ` 
    SELECT
        CONFIG
    FROM
        UPDATERPRODCONFIG
    WHERE
        CDPRODUCTID = :CDPRODUCTID
        AND NRVERSAO <> NVL(:NRVERSAO, 'X.X.X')
        AND CONFIG IS NOT NULL
    ORDER BY
      DTINCLUSAO DESC
  `;

  this.SET_UPDATE_PROD_CONFIG = `
    UPDATE 
      UPDATERPRODCONFIG
    SET
      CONFIG = :CONFIG    
    WHERE
      CDPRODUCTID = :CDPRODUCTID AND 
      NRVERSAO = :NRVERSAO
`;

  this.GET_UPDATE_PRODUCT_LAST_RECORD  = ` 
    SELECT 
      * 
    FROM 
      UPDATERPRODVERSAO      
    WHERE 
      UPDATERPRODID = :UPDATERPRODID AND
      DTINCLUSAO IS NOT NULL
      ORDER BY DTINCLUSAO DESC
      FETCH FIRST 1 ROW ONLY
  `;

  this.SET_NEW_VERSION_UPDATERPRODVERSAO = `
    INSERT INTO 
      UPDATERPRODVERSAO
        (
          UPDATERPRODID, 
          NRVERSAO, 
          DSURL, 
          DSDIRETORIO, 
          IDATIVO, 
          IDDOWNLOADVER, 
          IDAPROVVER, 
          DTINCLUSAO, 
          DTULTATU
        )
      VALUES 
        (
          :UPDATERPRODID, 
          :NRVERSAO, 
          :DSURL, 
          :DSDIRETORIO, 
          :IDATIVO, 
          :IDDOWNLOADVER, 
          :IDAPROVVER,         
           SYSDATE,
           SYSDATE
        )
  `;            

  this.INSERT_UPDATER_PROD_CONFIG = `
    INSERT INTO
      UPDATERPRODCONFIG
      (
        CDPRODUCTID,
        NRVERSAO,
        CONFIG,
        DTINCLUSAO, 
        DTULTATU
      )
      VALUES
      (
        :CDPRODUCTID,
        :NRVERSAO,
        :CONFIG,
        SYSDATE,
        SYSDATE
      )
  `;

  this.REMOVE_HISTORY_EVENTS = `
  DELETE FROM
    UPDATERHISTORICO
  WHERE
    CDFILIAL = :CDFILIAL
    AND CDPRODUCTID = :CDPRODUCTID
  `;

  this.REMOVE_EVENTS_BY_UNIT_AND_PRODUCT = `
  DELETE FROM UPDATEREVENTOS
  WHERE
    CDFILIAL = :CDFILIAL
    AND CDPRODUCTID = :CDPRODUCTID
  `;

  this.UPDATE_PACKAGE_URL = `
    UPDATE
      UPDATERPRODVERSAO
    SET
      DSURL = :DSURL,
      DSDIRETORIO = :DSDIRETORIO,
      DTULTATU = SYSDATE
    WHERE
      UPDATERPRODID = :UPDATERPRODID
      AND NRVERSAO = :NRVERSAO
  `;

  this.UPDATE_PRODFILI_VERSION = `
      UPDATE 
        UPDATERPRODFILI
      SET
        NRVERSAOATUAL = :P_NRVERSAO,
        DTULTATU = SYSDATE
      WHERE
        ID = :P_ID
  `;

  this.GET_PRODUCTS_VERSION = `
    SELECT
      PF.ID AS PRODFILIID,
      F.CDFILIAL || ' - ' || F.NMFILIAL AS FILIAL,
      F.CDFILIAL,
      F.NMFILIAL,
      P.ID AS PRODUCTID,
      P.NMPRODUCT,
      CASE
          WHEN PF.NRVERSAOATUAL IS NULL THEN
              'INSTALACAO_PENDENTE'
          ELSE
              'INSTALADO'
      END AS IDSTEVENTO,
      DECODE(PF.NRVERSAOATUAL, NULL, '-', PF.NRVERSAOATUAL) AS NRVERSAOATUAL,
      TO_CHAR(PF.DTULTATU, 'DD/MM/YYYY HH24:MI') AS DTULTATU,
      OP.CDOPERADOR,
      OP.NMOPERADOR
    FROM
      UPDATERPRODFILI PF
      INNER JOIN UPDATERPRODUTOS  P ON ( PF.UPDATERPRODID = P.ID )
      INNER JOIN FILIAL           F ON ( PF.CDFILIAL = F.CDFILIAL )
      LEFT JOIN (
          SELECT
              CDFILIAL,
              CDPRODFILIID,
              CDPRODUCTID,
              MAX(CDEVENTO) AS CDEVENTO
          FROM
              UPDATEREVENTOS
          WHERE
              IDSTEVENTO = 'FINALIZADO'
          GROUP BY
              CDFILIAL,
              CDPRODUCTID,
              CDPRODFILIID
      ) EV 
      ON ( 
              EV.CDPRODUCTID = P.ID
          AND EV.CDPRODFILIID = PF.ID
          AND EV.CDFILIAL = PF.CDFILIAL
      )
      LEFT JOIN UPDATERHISTORICO HIST 
          ON  HIST.CDEVENTO = EV.CDEVENTO
          AND HIST.IDSTEVENTO = 'FINALIZADO'
      LEFT JOIN OPERADOR OP 
          ON OP.CDOPERADOR = HIST.CDOPERADOR
    WHERE
      PF.CDFILIAL = :P_CDFILIAL
    ORDER BY
      P.NMPRODUCT ASC
  `;
}

  async getEventsByIdAndProduct(params) {
    return OracleService.fetchAssoc(this.GET_PRODUCT_UPDATER_EVENT_BY_EVENT_ID, params);
  };

  async getVersion(params) {
    return OracleService.fetchAssoc(this.GET_VERSION, params);
  };

  getCheckIfVersionIsInstalled(params) {
    return OracleService.connect().then(sql => {    
      return OracleService.fetchAssoc(this.ARRIVE_CASHIER_VERSION, params);
    });
  };

  async getEventHistory(params) {
      return OracleService.fetchAssoc(this.GET_UPDATER_HISTORY_BY_EVENT_ID, params);
  };

  getConfigIniHistoryByUnit(params) {
    return OracleService.connect().then(sql => {
      return OracleService.fetchAssoc(this.GET_CONFIG_INI_HISTORY_BY_UNIT, params);
    });
  };

  getProductPendingEvents(params) {
    return OracleService.connect().then(sql => {
      return OracleService.fetchAssoc(this.GET_PRODUCT_PENDING_EVENTS, params);
    });
  };

  getProductInProgressEvents(params) {
    return OracleService.connect().then(sql => {
      return OracleService.fetchAssoc(this.GET_PRODUCT_IN_PROGRESS_EVENTS, params);
    });
  };

  async getEvents(params) {
    return OracleService.fetchAssoc(this.GET_EVENTS_BY_UNIT, params);
  };
  
  async getEventsInProgress(params) {
    return OracleService.fetchAssoc(this.GET_IN_PROGRESS_EVENTS, params);
  };

  async getEventsInProgressByProduct(params) {
    return OracleService.fetchAssoc(this.GET_IN_PROGRESS_EVENTS_PRODUCT, params);
  }

  async getPendingEvents() {
    return OracleService.fetchAssoc(this.GET_PENDING_EVENTS);
  };

  getPendingEventsByCasher(params) {
    return OracleService.connect().then(sql => {
      return OracleService.fetchAssoc(this.GET_PENDING_EVENTS_BY_CASHIER, params);
    });
  };

  async getUpdaterPendingEvents(params) {
    return OracleService.fetchAssoc(this.GET_UPDATER_PENDING_EVENTS, params);
  }

  getDefaultOperator(params) {
    return OracleService.connect().then(sql => {
      return OracleService.fetchAssoc(this.GET_DEFAULT_OPERATOR, params);
    });
  };

  getDefaultOperatorFromSalesman(params) {
    return OracleService.connect().then(sql => {
      return OracleService.fetchAssoc(this.GET_DEFAULT_OPERATOR_FROM_SALESMAN, params);
    });
  };

  getConfigIniByUnit(params) {
    return OracleService.connect().then(() => {
      return OracleService.fetchAssoc(this.GET_CONFIG_INI_BY_UNIT, params);
    });
  }

  getServiceMonitorConfigurationByUnit(params) {
    return OracleService.connect().then(() => {
      return OracleService.fetchAssoc(this.GET_SERVICE_MONITOR_CONFIGURATION_BY_UNIT, params);
    });
  }

  updateServiceMonitorConfigurationByUnit(params) {
    return OracleService.execute(this.UPDATE_SERVICE_MONITOR_CONFIGURATION_BY_UNIT, params);
  }
  
  getUpdaterProdConfig(params) {
    return OracleService.connect().then(() => {
      return OracleService.fetchAssoc(this.GET_UPDATER_PROD_CONFIG, params);
    });
  }
  
  getUpdaterProdConfigLastUpdate(params) {
    return OracleService.connect().then(() => {
      return OracleService.fetchAssocSingleResult(this.GET_UPDATER_PROD_CONFIG_LAST_UPDATE, params);
    });
  }
  
  setUpdateProdConfig(params){
    return OracleService.execute(this.SET_UPDATE_PROD_CONFIG, params);
  };

  async insertHistory(params) {
    return OracleService.execute(this.APPEND_UPDATER_HISTORY, params);
  };
  
  appendConfigIniHistory(params) {
    return OracleService.execute(this.APPEND_CONFIG_INI_HISTORY, params);
  };

  async insertEvent(params) {
    return OracleService.execute(this.INSERT_PENDING_EVENT, params);
  };

  insertConfigIni(params) {
    return OracleService.execute(this.INSERT_CONFIG_INI, params, [{name: 'ID', type: "number"}])
  };

  updateEventStatus(params) {
    OracleService.execute(this.UPDATE_EVENT_IN_PROGRESS, params);
  };

  updateConfigIniByUnit(params) {
    return OracleService.execute(this.UPDATE_CONFIG_INI_BY_UNIT, params);
  };

  updatePackageURL(params) {
    return OracleService.execute(this.UPDATE_PACKAGE_URL, params);
  };
  
  async cancelUnitEvents(params) {
    return OracleService.execute(this.CANCEL_UNIT_EVENTS, params);
  };

  getUpdaterVersao() {
    return OracleService.connect().then(() => {
      return OracleService.fetchAssoc(this.GET_UPDATER);
    });
  }

  getUpdaterVersaoAtual(params) {
    return OracleService.connect().then(() => {
      return OracleService.fetchAssoc(this.GET_UPDATER_V_ATUAL, params);
    });
  }

  getUpdaterVersaoAll(params) {
    return OracleService.connect().then(() => {
      return OracleService.fetchAssoc(this.GET_UPDATER_V_ALL, params);
    });
  }

  /**
   * @author Daniel Souza <daniel.souza@teknisa.com>
   * @param {*} params {cdfilial: 9999}
   * @returns Lista de Sistemas a serem atualizados na filial cdFilial
   */
  getVersaoDisponivel(params) {
    return OracleService.getNewConnection().then(() => {
      return OracleService.fetchAssoc(this.GET_UPDATER_ESPERA_ALL, params);
    });
  }

  /**
   * @author Daniel Souza <daniel.souza@teknisa.com>
   * @param {*} params {cdfilial: 9999}
   * @returns Lista de Sistemas a serem atualizados na filial cdFilial
   */
   getVersaoDowngrade(params) {
    return OracleService.getNewConnection().then(() => {
      return OracleService.fetchAssoc(this.GET_DOWNGRADE_ESPERA_ALL, params);
    });
  }

  updateVersaoDisponivel(params) {
    return OracleService.getNewConnection().then(() => {
      return OracleService.execute(this.UPDATE_UPDATER, params);
    });
  }

  updateVersaoAtual(params) {
    return OracleService.getNewConnection().then(() => {
      return OracleService.execute(this.UPDATE_UPDATERSISTEMA_ATUAL, params);
    });
  }

  setUpdaterVersao(params) {
    return OracleService.getNewConnection().then(() => {
      return OracleService.execute(this.SET_UPDATERVERSAO, params);
    });
  }

  setDowngradeVersao(params) {
    return OracleService.getNewConnection().then(() => {
      return OracleService.execute(this.SET_DOWNGRADEVERSAO, params);
    });
  }

  setUpdaterSistemas(params, connection) {
    return OracleService.execute(this.SET_UPDATERSISTEMAS, params, [], connection);
  }

  setUpdaterSistemasCanceled(params, connection) {
    return OracleService.execute(this.SET_UPDATERSISTEMAS_CANCELED, params, [], connection);
  }

  getFullUpdaterSistemas(connection) {
    return OracleService.execute(this.GET_FULL_UPDATERSISTEMAS, [], [], connection);
  }

  /**
   * @author Daniel Souza <daniel.souza@teknisa.com>
   * @param {*} params {cdfilial: 9999}
   * @returns Retorna se o sistema já tem um pedido de update
   */
   getRegistroUpdate(params) {
    return OracleService.getNewConnection().then(() => {
      return OracleService.fetchAssoc(this.GET_UPDATERVERSAO, params);
    });
  }

  getConfigIni(params){
    return OracleService.getNewConnection().then(() => {
      return OracleService.fetchAssoc(this.GET_CONFIG_INI, params);
    });
  }
  getUpdateLastRecordProduct(params) {
    return OracleService.connect().then(() => {
      return OracleService.fetchAssoc(this.GET_UPDATE_PRODUCT_LAST_RECORD, params);
    });
  }

  setUpdateProdVersion(params) {
    return OracleService.execute(this.SET_NEW_VERSION_UPDATERPRODVERSAO, params);
  }
  
  insertUpdaterProdConfig(params){
    return OracleService.execute(this.INSERT_UPDATER_PROD_CONFIG, params);
  }

  removeHistoryEvents(params) {
    return OracleService.execute(this.REMOVE_HISTORY_EVENTS, params);
  }

  removeEventsByUnitAndProduct(params){
    return OracleService.execute(this.REMOVE_EVENTS_BY_UNIT_AND_PRODUCT, params);
  }

  async updateProdFiliVersion(params) {
    return OracleService.execute(this.UPDATE_PRODFILI_VERSION, params);
  }

  async getProductsVersion(params) {
    return OracleService.fetchAssoc(this.GET_PRODUCTS_VERSION, params);
  }
}

module.exports = new updaterModel();
