const OracleService = require('../service/OracleService');

class updaterProdVersaoModel 
{  
  getUpdaterProdVersao(updaterProdId) {
    const sql = `
      SELECT ID, NRVERSAO, DSURL, DSDIRETORIO
        FROM UPDATERPRODVERSAO
       WHERE UPDATERPRODID = :UPDATERPRODID
    `;

    const params = {
      UPDATERPRODID: updaterProdId
    };

    return OracleService.fetchAssoc(sql, params);
  }

  getUpdaterProdVersaoToDownload() {
    const sql = `
    SELECT V.ID, V.NRVERSAO, V.DSURL, V.DSDIRETORIO
      FROM UPDATERPRODVERSAO V
      JOIN UPDATERPRODUTOS P ON P.ID = V.UPDATERPRODID
     WHERE P.IDATIVO = 'S'
    `;

    return OracleService.fetchAssoc(sql);
  }

  getUpdaterProdVersaoByProduct(productId, useVersionFilter) {
    const filter = useVersionFilter === 'true' ? " AND V.IDAPROVVER = 'S'" : "";

    const sql = `
      SELECT 
        V.ID, 
        V.NRVERSAO, 
        V.DSURL, 
        V.DSDIRETORIO, 
        V.IDDOWNLOADVER, 
        P.ID AS PRODUCT_ID
      FROM 
        UPDATERPRODVERSAO V
        JOIN UPDATERPRODUTOS P ON P. ID = V.UPDATERPRODID
      WHERE 
        P.ID = :ID
        ${filter}
      ORDER BY 
        V.NRVERSAO DESC
    `;

    const params = {
      ID: productId,
    }

    return OracleService.fetchAssoc(sql, params);
  }

  insertUpdaterProdVersao(params) {
    const sql = `
      INSERT INTO UPDATERPRODVERSAO
                ( UPDATERPRODID,  NRVERSAO,  DSURL,  DSDIRETORIO, DTINCLUSAO, DTULTATU)
         VALUES (:UPDATERPRODID, :NRVERSAO, :DSURL, :DSDIRETORIO, SYSDATE,    SYSDATE)
    `;
    return OracleService.execute(sql, params);
  }

  updateUpdaterProdVersaoStatus(params) {
    const sql = `
      UPDATE UPDATERPRODVERSAO
         SET IDDOWNLOADVER = :IDDOWNLOADVER
       WHERE ID = :ID
    `;
    return OracleService.execute(sql, params);
  }

  getUpdaterProdVersaoConfig() {
    const sql = `
      SELECT 
        P.ID AS PRODUCT_ID, 
        P.NMPRODUCT, 
        V.ID, 
        V.NRVERSAO, 
        V.IDDOWNLOADVER, 
        V.IDAPROVVER
      FROM 
        UPDATERPRODVERSAO V
        JOIN UPDATERPRODUTOS P ON P.ID = V.UPDATERPRODID
      WHERE 
        P.IDATIVO = 'S'
    `;

    return OracleService.fetchAssoc(sql);
  }
  
  updateUpdaterProdVersaoAprov(params) {
    const sql = `
      UPDATE UPDATERPRODVERSAO
         SET IDAPROVVER = :IDAPROVVER
       WHERE ID = :ID
    `;
    return OracleService.execute(sql, params);
  }
}

module.exports = new updaterProdVersaoModel();
