import { ComponentRender } from '@zeedhi/common';
import { IDropdownWarning, IDropdownWarningEvents } from './interfaces';

export class DropdownWarning extends ComponentRender implements IDropdownWarning {
    public items?: Array<any> = [];

    public badge?: number;

    public isVisible: boolean = false;

    public isVisibleBadge?: boolean = false;

    public hideEmptyBadge?: boolean = true;

    public dataValue!: string;

    public dataText!: string;

    public componentName!: string;

    public events!: IDropdownWarningEvents;

    constructor(props: IDropdownWarning) {
        super(props);
        this.componentName = this.getInitValue('name', props.name, this.componentName);
        this.items = this.getInitValue('items', props.items, this.items);
        this.badge = this.getInitValue('badge', props.badge, this.badge);
        this.dataValue = this.getInitValue('dataValue', props.dataValue, this.dataValue);
        this.dataText = this.getInitValue('dataText', props.dataText, this.dataText);
        this.isVisibleBadge = this.getInitValue('isVisibleBadge', props.isVisibleBadge, this.isVisibleBadge);
        this.hideEmptyBadge = this.getInitValue('hideEmptyBadge', props.hideEmptyBadge, this.hideEmptyBadge);
        this.createAccessors();
    }

    public clearItems() {
        this.items = [];
        this.setBadge(0);
        this.handleEmptyBadge();
        this.handleVisibility();
    }

    private handleEmptyBadge() {
        if (this.hideEmptyBadge && !this.badge) {
            this.hideBadge();
        }
    }

    private handleVisibility() {
        this.isVisible = !!this.items?.length;
    }

    public getItems(): Array<any> {
        return this.items || [];
    }

    public setItems(items: Array<any>) {
        this.items = items;
        this.setBadge(items.length);
        this.handleEmptyBadge();
        this.handleVisibility();
    }

    public addItem(item: any): void {
        this.items?.push(item);
        this.setBadge(this.items?.length || 0);
        this.handleEmptyBadge();
        this.handleVisibility();
    }

    public showBadge() {
        this.isVisibleBadge = true;
    }

    public hideBadge() {
        this.isVisibleBadge = false;
    }

    public setBadge(value: number) {
        this.badge = value;
        if (value > 0) {
            this.showBadge();
        }
    }

    public removeItem(itemId: any) {
        const idxToRemove = (this.items || []).findIndex((item: any) => item[this.dataValue] === itemId);
        if (idxToRemove > -1) {
            this.items?.splice(idxToRemove, 1);
            this.badge = (this.badge || 1) - 1;
        }
        this.handleEmptyBadge();
        this.handleVisibility();
    }
}
