import { IEventParam, Http, Metadata } from '@zeedhi/core';
import { AlertService, DialogService, Form, LoadingService } from '@zeedhi/common';

export class LoginController {
  public messageValue?:string;

  login:Array<Form> = [];

  boddys:any = [];

  /** */
  public gridMounted({ component }: IEventParam<Form>) {
    this.login.push(component);
  }

  public logout() {
    DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      title: 'Confirmação',
      text: 'Deseja realmente sair do sistema?',
      buttons: [
        { name: 'yes', component: 'ZdButton', label: 'Sim', events: this.doLogout, outline: true },
        { name: 'no', component: 'ZdButton', label: 'Não', events: this.backLogout },
      ],
    });
  }

  private doLogout: any = {
    click: () => {
      localStorage.removeItem('NMOPERADOR');
      localStorage.removeItem('CDOPERADOR');
      localStorage.removeItem('IDOPERACESSO');
      localStorage.removeItem('TOKEN');
      window.location.reload();
    },
  };

  private backLogout: any = {
    click: () => DialogService.hide(),
  };

  public logins() {
    const form = Metadata.getInstance<Form>('login-form');
    this.loginRequestMaster({
      email: form.value.email,
      password: form.value.password,
    });
  }

  public loginJson(email:string, pass:string) {
    this.boddys = {
      disableLoader: false,
      filter: [{
        name: 'EMAIL',
        operator: '=',
        value: email,
      },
      {
        name: 'PASSWORD',
        operator: '=',
        value: pass,
      },
      {
        name: 'PRODUCT_ID',
        operator: '=',
        value: 1776,
      },
      {
        name: 'REQUESTER_URL',
        operator: '=',
        value: 'https://retail.teknisa.cloud/login/#/login#authentication',
      },
      {
        name: 'ATTEMPTS',
        operator: '=',
        value: 1,
      },
      {
        name: 'SHOW_FULL_OPERATOR',
        operator: '=',
        value: true,
      },
      {
        name: 'HASH',
        operator: '=',
        value: null, // 'wolrm8i1one',
      },
      {
        name: 'SESSION_CHANGE',
        operator: '=',
        value: false,
      },
      {
        name: 'KEEP_CONNECTED',
        operator: '=',
        value: 'N',
      }],
      page: 1,
      requestType: 'FilterData',
      origin: {
        containerName: 'AUTHENTICATION',
        widgetName: 'LOGIN',
        containerLabel: 'Autenticação',
        widgetLabel: 'Login',
      },
    };
  }

  public async loginRequestMaster(data: any) {
    LoadingService.show();
    const response = await Http.post('/login', data);

    if (response.data.CDOPERADOR !== undefined) {
      localStorage.setItem('CDOPERADOR', response.data.CDOPERADOR);
      localStorage.setItem('IDOPERACESSO', response.data.IDOPERACESSO);
      localStorage.setItem('NMOPERADOR', response.data.NMOPERADOR);
      localStorage.setItem('TOKEN', response.data.TOKEN);
      document.location.href = document.location.origin;
    } else {
      AlertService.show({
        name: 'buttons-alert',
        text: response.data,
        color: 'error',
        right: true,
        top: true,
        showDismiss: true,
      });
    }

    LoadingService.hide();
  }

  /**
   * Método executado logo antes do grid ser destruído
   */
  public gridBeforeDestroy() {

  }
}
