import AlertHelper from '@/helpers/AlertHelper';
import DataSourceHelper from '@/helpers/DataSourceHelper';
import {
  DialogService,
  LoadingService,
  Modal,
  ModalService,
} from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';
import { ZdButton, ZdProgress, ZdText } from '@zeedhi/vuetify';
import { DropdownWarning } from '@/components/dropdown-warning/common/DropdownWarning';
import ModalParameters from './ModalParametersController';

export class ModalMultipleInstallController {
  static modal?: Modal | null;

  static params?: any;

  static socket?: any;

  static opened = false;

  static installId: string | null = null;

  static warningMessages: any = {
    DISCONNECTED: {
      code: 'DISCONNECTED',
      message: 'Unidade desconectada',
    },
    INSTALLED_VERSION: {
        code: 'INSTALLED_VERSION',
        message: 'A versão já está instalada',
    },
    NO_EMPTY_SLOT: {
      code: 'NO_EMPTY_SLOT',
      message: 'Não há slots de versão disponíveis.',
    },
  };

  static updatesInProgress = 0;

  static updatesFinished = 0;

  static error = false;

  public static async create() {
    this.modal = await ModalService.createFromJson('modal-multiple-install', true);
  }

  public static show() {
    this.modal?.show();
    this.opened = true;
  }

  public static setSocket(socket: any) {
    this.socket = socket;
  }

  public static setParams(params: any) {
    this.params = params;
  }

  public static getParams() {
    return this.params;
  }

  public hide(): void {
    ModalMultipleInstallController.hide();
  }

  public static hide(): void {
    this.modal?.hide();
    this.opened = false;
  }

  public static destroy(): void {
    this.modal?.destroy();
  }

  private static getInstance<T>(name: string) {
    try {
      const instance = Metadata.getInstance<T>(name);
      return instance || null;
    } catch (error) {
      return null;
    }
  }

  private getDropdown(cdfilial: string): DropdownWarning | null {
    return ModalMultipleInstallController.getDropdown(cdfilial);
  }

  private static getDropdown(cdfilial: string): DropdownWarning | null {
    try {
      const dropdown: DropdownWarning | null = this.getInstance<DropdownWarning>(`dropdownWarning_${cdfilial}`);
      return dropdown;
    } catch (error) {
      return null;
    }
  }

  private static addDropdownWarning(
    cdfilial: string,
    warningNames: Array<string> | string,
  ) {
    const dropdown: any = this.getDropdown(cdfilial);
    const warningKeys = Array.isArray(warningNames) ? warningNames : [warningNames];
    warningKeys.forEach((msgKey: string) => {
      const message = ModalMultipleInstallController.warningMessages[msgKey];
      if (message && dropdown) {
        dropdown.addItem(message);
      }
    });
  }

  private setDropdownWarning(cdfilial: string, messages: Array<any>) {
    const dropdown: DropdownWarning | null = this.getDropdown(cdfilial);
    if (dropdown) {
      dropdown.setItems(messages);
      return !!messages.length;
    }
    return false;
  }

  private static removeDropdownWarning(cdfilial: string, messageCode: string) {
    const list: DropdownWarning | null = this.getDropdown(cdfilial);
    if (list) {
      list.removeItem(messageCode);
    }
  }

  private getParametersBtn(): ZdButton | null {
    try {
      const btn: ZdButton = Metadata.getInstance<ZdButton>(
        'parametersBtnMultiInstall',
      );
      return btn || null;
    } catch (error) {
      return null;
    }
  }

  private toggleHintButtonParameters(visibility: boolean) {
    const hint = document.getElementById('hintDisabledParameters');
    if (!hint) return;
    if (visibility) {
      hint.classList.remove('invisible-element');
    } else {
      hint.classList.add('invisible-element');
    }
  }

  private toggleParametersBtn(enabled: boolean) {
    try {
      const parametersBtn = this.getParametersBtn();
      if (parametersBtn) {
        parametersBtn.disabled = !enabled;
        this.toggleHintButtonParameters(!enabled);
      }
    } catch (error) {
      return null;
    }
  }

  public changeVersion({ value }: any) {
    if (value && value.NRVERSAO) {
      this.fillGridWarning(value.NRVERSAO);
      this.toggleParametersBtn(true);
    } else {
      this.clearAllWarnings();
      this.toggleParametersBtn(false);
    }
  }

  private setVersionMessage(
    row: any,
    version: string | null,
    messagesCollection: Array<String>,
    IDUTMULTVER: string,
  ) {
    const isInstalledVersion = version && row.VERSOES.includes(version);
    const { INSTALLED_VERSION, NO_EMPTY_SLOT } = this.getWarningMessages();
    if (isInstalledVersion) {
      messagesCollection.push(INSTALLED_VERSION);
    } else if (row.QTEMPTYSLOTS === 0 && IDUTMULTVER === 'S') {
      messagesCollection.push(NO_EMPTY_SLOT);
    }
  }

  private getWarningMessages() {
    return ModalMultipleInstallController.getWarningMessages();
  }

  private static getWarningMessages() {
    return this.warningMessages;
  }

  private setConnectionMessage(row: any, messagesCollection: Array<string>) {
    const isActiveConn = row.CONNECTION_STATUS;
    const { DISCONNECTED } = this.getWarningMessages();
    if (!isActiveConn) {
      messagesCollection.push(DISCONNECTED);
    }
  }

  private fillGridWarning(NRVERSAO: any = null) {
    const grid: any = this.getMultiInstallGrid();
    const { IDUTMULTVER } = this.getModalParams();
    grid.datasource.data.forEach((row: any) => {
      const warningMessages: Array<string> = [];
      this.setConnectionMessage(row, warningMessages);
      this.setVersionMessage(row, NRVERSAO, warningMessages, IDUTMULTVER);
      row.WARNING = this.setDropdownWarning(row.CDFILIAL, warningMessages);
    });
  }

  public getConnectionColor({ row }: any) {
    return row.CONNECTION_STATUS ? 'green' : 'red';
  }

  public async onMountedProduct({ component: productSelect }: any) {
    const { ID }: any = ModalMultipleInstallController.params;
    await productSelect.datasource.get();
    productSelect.setValue(ID);
  }

  public onMountedVersion({ component: versionSelect }: any) {
    const { ID }: any = ModalMultipleInstallController.params;
    versionSelect.datasource.filter = {
      CDPRODUCTID: ID,
    };
  }

  public static getVersionSelect() {
    try {
      const versionSelect: any = Metadata.getInstance('availableVersion');
      return versionSelect || null;
    } catch (error) {
      return null;
    }
  }

  public getVersionSelect() {
    return ModalMultipleInstallController.getVersionSelect();
  }

  public static getProductSelect() {
    try {
      const productSelect: any = Metadata.getInstance('NMPRODUCT');
      return productSelect || null;
    } catch (error) {
      return null;
    }
  }

  public getProductSelect() {
    return ModalMultipleInstallController.getProductSelect();
  }

  public getMultiInstallGrid() {
    return ModalMultipleInstallController.getMultiInstallGrid();
  }

  public static getMultiInstallGrid() {
    try {
      const grid: any = Metadata.getInstance('multi-install-grid');
      return grid || null;
    } catch (error) {
      return null;
    }
  }

  private dataHandler(row: any) {
    const VERSIONS = row.VERSOES.split(';');
    row.VERSOES = VERSIONS.join('\n');
    row.ARRAY_VERSIONS = VERSIONS;
    row.ARRAY_PRODFILIIDS = row.PRODFILIIDS.split(';');
    row.QTEMPTYSLOTS = (row.VERSOES.match(/-/g) || []).length;
    row.FILIAL = `${row.CDFILIAL} - ${row.NMFILIAL}`;
    const hasInstalledVersion = row.QTEMPTYSLOTS < VERSIONS.length;
    row.IDSTEVENTO = hasInstalledVersion && !row.IDSTEVENTO
        ? 'INSTALADO'
        : row.IDSTEVENTO || 'INSTALACAO_PENDENTE';
  }

  private clearWarning(cdfilial: string) {
    const dropdown: DropdownWarning | null = this.getDropdown(cdfilial);
    dropdown?.clearItems();
  }

  private clearAllWarnings() {
    const grid = this.getMultiInstallGrid();
    const dataset = grid.datasource.data;
    dataset.forEach(({ CDFILIAL }: any) => this.clearWarning(CDFILIAL));
  }

  public async changeProduct({ value }: any) {
    try {
      const versionSelect: any = this.getVersionSelect();
      const grid = this.getMultiInstallGrid();
      if (!versionSelect) return;
      versionSelect.setValue(null);
      const CDPRODUCTID = value?.ID;
      versionSelect.datasource.filter = {
        CDPRODUCTID,
      };
      ModalMultipleInstallController.params.ID = value;
      const { UNITS = [] } = ModalMultipleInstallController.params;
      LoadingService.show();
      await versionSelect.datasource.get();
      const { data: dataset } = await Http.post('/updaterProductMultiInstall', {
        PRODUCT_ID: CDPRODUCTID,
        UNITS: [...new Set(UNITS)],
      });
      dataset.data.forEach(this.dataHandler.bind(this));
      this.clearAllWarnings();
      grid.datasource.updateData(dataset.data);
    } catch (error) {
      console.error(error);
    } finally {
      LoadingService.hide();
    }
  }

  private getModalParams() {
    return ModalMultipleInstallController.getModalParams();
  }

  private static getModalParams() {
    const productSelect = this.getProductSelect();
    const versionSelect = this.getVersionSelect();
    const params = { CDPRODUCTID: '', NRVERSAO: '', IDUTMULTVER: '' };
    if (productSelect && versionSelect) {
      params.CDPRODUCTID = productSelect.value?.ID;
      params.IDUTMULTVER = productSelect.value?.IDUTMULTVER;
      params.NRVERSAO = versionSelect.value?.NRVERSAO;
    } else {
      throw new Error('ERR_INVALID_COMPONENTS');
    }
    return params;
  }

  public editParams() {
    const params = this.getModalParams();
    ModalParameters.removeEvent('afterSave');
    ModalParameters.show(params);
  }

  private async getProdConfig(
    UPDATERPRODID: string | number,
    NRVERSAOATUAL: string,
  ) {
    return Http.post('/updater/getProdConfig', {
      UPDATERPRODID,
      NRVERSAOATUAL,
    });
  }

  private getFormattedProdConfig(config: any) {
    const formattedConfig = Object.entries(config).map(
      ([key, value]) => `<strong>${key}: </strong>${value}<br>`,
    );
    return formattedConfig.join('');
  }

  private static getEmptySlot(row: any, IDUTMULTVER: string) {
    if (IDUTMULTVER === 'N') {
      return {
        PRODFILIID: row.PRODFILIIDS,
        CURRENTVERSION: row.VERSOES === '-' ? null : row.VERSOES,
      };
    }
    const idxEmptySlot = row.ARRAY_VERSIONS.findIndex('-');
    return {
      PRODFILIID: row.ARRAY_PRODFILIIDS[idxEmptySlot],
      CURRENTVERSION: null,
    };
  }

  private static getInfoToInstall() {
    const grid = this.getMultiInstallGrid();
    const { CDPRODUCTID, NRVERSAO, IDUTMULTVER } = this.getModalParams();
    const UNITS: Array<any> = [];
    grid.datasource.data.forEach((row: any) => {
      const isInstalledVersion = row.VERSOES.includes(NRVERSAO);
      const hasActiveConnection = row.CONNECTION_STATUS;
      const hasAvailableSlot = row.QTEMPTYSLOTS > 0 || IDUTMULTVER === 'N';
      const availableUnit = ['INSTALACAO_PENDENTE', 'INSTALADO', 'ERROR'].includes(row.IDSTEVENTO);
      if (!isInstalledVersion && hasActiveConnection && hasAvailableSlot && availableUnit) {
        const emptySlot = this.getEmptySlot(row, IDUTMULTVER);
        UNITS.push({
          ...emptySlot,
          CDFILIAL: row.CDFILIAL,
        });
      }
    });
    return {
      CDOPERADOR: localStorage.getItem('CDOPERADOR'),
      TYPE: 'INSTALL',
      INSTALLVERSION: NRVERSAO,
      CDPRODUCTID,
      DEVOPSID: CDPRODUCTID,
      NRVERSAO,
      IDUTMULTVER,
      CREATENEWVERSION: IDUTMULTVER,
      INSTALLID: null,
      UNITS,
    };
  }

  private static async executeInstall() {
    const paramsToInstall = this.getInfoToInstall();
    LoadingService.show();
    this.error = false;
    try {
      const result = await Http.post('/updater/multiInstallVersion', paramsToInstall);
      if (result.data && result.data.error) {
        AlertHelper.showError(result.data.message);
      } else {
        const { INSTALLID } = result.data;
        paramsToInstall.INSTALLID = INSTALLID;
        this.installId = INSTALLID;
        this.socket?.emit('execUpdaterMultiInstall', paramsToInstall);
        this.updatesInProgress = paramsToInstall.UNITS.length;
        this.toggleProgressFooter(true);
        this.updateFinishedUnits(true);
        AlertHelper.showSuccess('Requisição de instalação/atualização enviada com sucesso!');
      }
    } catch (error) {
      AlertHelper.showError(error);
    } finally {
      LoadingService.hide();
    }
  }

  private getUnitsNumberWithWarning() {
    const warningBtn = [...document.querySelectorAll('.warning-btn:not(.d-none)')];
    return warningBtn.filter((button: any) => button.style.display !== 'none').length;
  }

  private async getEventsInProgress(params: any) {
    try {
      const eventsInProgress = await Http.post('/getEventsInProgressByProduct', params);
      return eventsInProgress.data;
    } catch (error) {
      return error;
    }
  }

  public async showInstallConfirmDialog() {
    const { CDPRODUCTID, NRVERSAO } = this.getModalParams();

    if (!NRVERSAO) return AlertHelper.showAlert('Selecione a versão');

    const grid = this.getMultiInstallGrid();
    const { data } = grid.datasource;
    const unitsWithoutWarning: Array<string> = [];
    data.forEach((row: any) => {
      if (!row.WARNING) {
        unitsWithoutWarning.push(row.CDFILIAL);
      }
    });
    const eventsInProgress = await this.getEventsInProgress({
      PRODUCT_ID: CDPRODUCTID,
      UNITS: unitsWithoutWarning,
    });
    if (eventsInProgress.EVENTS?.length) {
      return AlertHelper.showAlert('Existem unidades selecionadas que estão com instalações/remoções em andamento. Tente novamente mais tarde.');
    }
    const { data: config } = await this.getProdConfig(CDPRODUCTID, NRVERSAO);
    const formattedConfig = this.getFormattedProdConfig(config.data);

    const hasParameters = !!formattedConfig;
    const textDialogPrefix = hasParameters
      ? `<p>Parâmetros do Projeto</p>\n${formattedConfig}`
      : '<strong>Projeto não possui parâmetros</strong>';
    const unitsNumberWithWarning = this.getUnitsNumberWithWarning();
    const unitsWithWarning = unitsNumberWithWarning > 0
      ? `<br><strong>* Número de filiais que não serão atualizadas: </strong>${unitsNumberWithWarning}<br>`
      : '';

    if (unitsNumberWithWarning === grid.datasource.data.length) {
      return AlertHelper.showAlert('Todas as unidades possuem inconsistências. Verifique-as na coluna "Ações" e após resolvê-las tente novamente.');
    }

    DialogService.show({
      name: 'install-dialog',
      type: 'warning',
      title: 'Instalação',
      text: `    
      ${textDialogPrefix}
      ${unitsWithWarning}
      <br>Deseja realmente instalar/atualizar esse sistema?                      
        `,
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          outline: true,
          events: {
            click: () => {
              DialogService.hide();
            },
          },
        },
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: () => {
              DialogService.hide();
              ModalMultipleInstallController.executeInstall();
            },
          },
        },
      ],
    });
  }

  private static handleWarningVersion(row: any, NRVERSAO: any) {
    const dropdown = this.getDropdown(row.CDFILIAL);
    const hasWarningVersion = dropdown?.getItems().find(({ code }: any) => code === 'INSTALLED_VERSION');
    if (row.VERSOES.includes(NRVERSAO)) {
      if (row.IDSTEVENTO === 'INSTALADO' && !hasWarningVersion) {
        dropdown?.addItem(this.getWarningMessages().INSTALLED_VERSION);
      }
    } else if (hasWarningVersion) {
      dropdown?.removeItem('INSTALLED_VERSION');
    }
  }

  public static toggleProgressFooter(visibility: boolean) {
    const progressFooter = document.getElementById('containerProgressLeftSlot');
    if (visibility) {
      progressFooter?.classList.remove('invisible-element');
    } else {
      progressFooter?.classList.add('invisible-element');
    }
  }

  private static isVisibleProgressFooter() {
    const progressFooter = document.getElementById('containerProgressLeftSlot');
    return !progressFooter?.classList.contains('invisible-element');
  }

  public static getPercentageProgress() {
    if (this.updatesInProgress === 0) return 0;
    return (this.updatesFinished / this.updatesInProgress) * 100;
  }

  public static getFormattedProgress() {
    return `${this.updatesFinished} / ${this.updatesInProgress}`;
  }

  public static getTextFinishedItems(): ZdText | null {
    try {
      return Metadata.getInstance<ZdText>('finishedItems');
    } catch (error) {
      return null;
    }
  }

  public static getProgressBar(): ZdProgress | null {
    try {
      return Metadata.getInstance<ZdProgress>('updateProgressBar');
    } catch (error) {
      return null;
    }
  }

  private static showMessageFinishUpdate() {
    if (this.error) {
      AlertHelper.showAlert('As atualizações foram finalizadas, porém ocorreram erros. Verifique o log das filiais.');
    } else {
      AlertHelper.showSuccess('As atualizações foram finalizadas com sucesso.');
    }
  }

  public static updateFinishedUnits(initial: boolean = false) {
    if (!this.isVisibleProgressFooter()) return;
    if (initial) {
      this.updatesFinished = 0;
    } else {
      this.updatesFinished += 1;
    }
    const textFinishedItems: ZdText | null = this.getTextFinishedItems();
    const progressBar = this.getProgressBar();
    if (textFinishedItems) {
      textFinishedItems.text = this.getFormattedProgress();
    }
    if (progressBar) {
      progressBar.value = this.getPercentageProgress();
      if (progressBar.value === 100) {
        this.showMessageFinishUpdate();
      }
    }
  }

  private static handleConnectionStatus(grid: any, connectionColumn: any, row: any) {
    grid.reapplyConditions(row, [connectionColumn]);
    if (row.CONNECTION_STATUS) {
      this.removeDropdownWarning(row.CDFILIAL, 'DISCONNECTED');
    } else {
      this.addDropdownWarning(row.CDFILIAL, 'DISCONNECTED');
    }
  }

  private static isEqualsStr(item1: any, item2: any) {
    return (item1 || '').toString() === (item2 || '').toString();
  }

  private static handleChangedVersion(row: any, rowInfo: any) {
    const idxProdFili = row.ARRAY_PRODFILIIDS.findIndex((PRODIFILID: any) => this.isEqualsStr(PRODIFILID, rowInfo.PRODFILIID));
    if (idxProdFili === -1 || !['INSTALADO', 'INSTALACAO_PENDENTE'].includes(row.IDSTEVENTO)) return;
    row.ARRAY_VERSIONS.splice(idxProdFili, 1, rowInfo.INSTALLVERSION || '-');
    row.VERSOES = row.ARRAY_VERSIONS.join('\n');
  }

  public static updateGridRow(
    rowInfo: any,
    newValues: Object | any,
    error: boolean = false,
  ) {
    if (!this.opened) return;
    const grid = this.getMultiInstallGrid();
    const {
      CDPRODUCTID,
      NRVERSAO,
    } = this.getModalParams();
    const {
      DEVOPSID,
      CDFILIAL,
      INSTALLID,
    } = rowInfo;
    const isChangeOfFiltredProd = !DEVOPSID || DEVOPSID.toString() === CDPRODUCTID.toString();
    const isCurrentInstall = !INSTALLID || INSTALLID === this.installId;
    if (!isChangeOfFiltredProd) return;
    if (error) {
      newValues.IDSTEVENTO = 'ERROR';
      this.error = true;
    }
    const UNITS = Array.isArray(CDFILIAL) ? CDFILIAL : [CDFILIAL];
    const connectionColumn = grid.columns.find((column: any) => column.name === 'CONNECTION');
    const rows = DataSourceHelper.updateGridRow(grid, 'CDFILIAL', UNITS, newValues, grid.datasource.data);
    if (['INSTALACAO_PENDENTE', 'INSTALADO', 'ERROR'].includes(newValues.IDSTEVENTO) && isCurrentInstall) {
      this.updateFinishedUnits();
    }
    if (connectionColumn) {
      rows?.forEach((row: any) => {
        this.handleConnectionStatus(grid, connectionColumn, row);
        this.handleChangedVersion(row, rowInfo);
        if (NRVERSAO) {
          this.handleWarningVersion(row, NRVERSAO);
        }
      });
    }
  }
}
