import AlertHelper from '@/helpers/AlertHelper';
import { LoadingService, Modal, ModalService } from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';

class ModalVersionController {
  private static modal: Modal | null;

  private static params: any;

  private static titleText: string;

  private static modalContent: any;

  constructor() {
    if (!ModalVersionController.modalContent) {
      Metadata.get('modal-version', true).then((modalContent: any) => {
          ModalVersionController.modalContent = modalContent;
          ModalVersionController.modal = ModalService.create(modalContent);
          this.bindEvents();
      });
    }
  }

  private bindEvents(events?: object | any) {
    const closeModal = this.getCloseButton();
    if (closeModal) {
      closeModal.events = {
        click: () => {
          if (events && typeof events.close === 'function') {
            events.close.apply(this, []);
          }
          this.close();
        },
      };
    }
  }

  public setTitle(titleText: string) {
    ModalVersionController.titleText = titleText;
  }

  public setParams(params: any) {
    ModalVersionController.params = params;
  }

  public getParams() {
    return ModalVersionController.params;
  }

  private createModalInstance() {
    if (!ModalVersionController.modal) {
        ModalVersionController.modal = ModalService.create(ModalVersionController.modalContent);
    }
  }

  private applyModalTitle() {
    const title = this.getTitle();
    if (!title) return;
    title.text = ModalVersionController.titleText;
  }

  private close() {
    ModalVersionController.modal?.hide();
    if (ModalVersionController.modal) {
        ModalService.delete(ModalVersionController.modal);
    }
    ModalVersionController.modal?.destroy();
    ModalVersionController.modal = null;
  }

  public show() {
    this.createModalInstance();
    this.applyModalTitle();
    ModalVersionController.modal?.show();
  }

  private getHeader() {
    const header = this.getChildElement('modal-header');
    return header;
  }

  private getCloseButton() {
    const header = this.getHeader();
    if (!header) return null;
    const closeButton = header.rightSlot.find((child: any) => child.name === 'closeModalButton');
    return closeButton;
  }

  private getTitle() {
    const header = this.getHeader();
    if (!header) return null;
    const title = header.leftSlot.find((child: any) => child.name === 'titleText');
    return title;
  }

  private getChildElement(name: any) {
    const element = ModalVersionController.modal?.children.find((child: any) => child.name === name);
    return element || null;
  }

  public async onMounted({ component: grid }: any) {
    try {
      const params = this.getParams();
      LoadingService.show();
      const productsVersion = await Http.get('/data/productsVersion', { params });
      const dataset = productsVersion.data.data;
      grid.datasource.updateData(dataset);
    } catch (error) {
      AlertHelper.showError(error);
    } finally {
      LoadingService.hide();
    }
  }
}

export { ModalVersionController };
export default new ModalVersionController();
