import AlertHelper from '@/helpers/AlertHelper';
import { SocketEvent } from '@/socket/socket-event';
import DataSourceHelper from '@/helpers/DataSourceHelper';
import { Modal, ModalService } from '@zeedhi/common';
import { Config, Http, IDictionary, Metadata } from '@zeedhi/core';
import { TekGrid, TekGridColumn, ITekGridColumn, IFilterPropsComponent, TekMemoryDatasource } from '@zeedhi/teknisa-components-common';
import { io, Socket } from 'socket.io-client';

export class MonitoringController {
  public socket?: Socket;

  public isVisibleModalColumn: boolean = false;

  private monitoringModal!: Modal;

  private cdfilial: any;

  private clickedAssConfig: any;

  private assConfig: any;

  private satColumnId: any;

  private columnsId: any = [];

  private socketArgs = {
    email: 'assistant@teknisa.com',
    password: 'nineninenine',
    slave: false,
  };

  public gridBeforeApplyFilter() {
    const { datasource } = Metadata.getInstance<TekGrid>('monitoring-grid');
    const filter = (datasource as TekMemoryDatasource).dynamicFilter;
    localStorage.setItem('LAST_FILTER_MONITORING', JSON.stringify(filter));
  }

  private addToolbarSwitchAction(grid: TekGrid) {
    const tagElement = grid.toolbarSlot.find((el: any) => el.name === 'monitoring-grid_notEditingSpan');

    if (!tagElement || !Array.isArray(tagElement.children)) {
      return;
    }
    tagElement.cssClass = 'toolbar-tag-actions';
    const switchAction = {
      name: 'switchMonitoringLayout',
      component: 'ZdSwitch',
      showHelper: false,
      cssClass: 'toolbar-switch-action',
      trueValue: 'Y',
      falseValue: 'N',
      label: 'Exibir número de erros',
      value: 'N',
      events: {
        change: ({ component }: any) => {
          const gridEl = document.getElementById(grid.name);
          if (!gridEl) return;
          if (component.value === 'Y') {
            gridEl.classList.add('expanded-layout');
          } else {
            gridEl.classList.remove('expanded-layout');
          }
          localStorage.setItem('USE_EXPANDED_LAYOUT', component.value);
        },
        onMounted: ({ component, element }: any) => {
          const value = localStorage.getItem('USE_EXPANDED_LAYOUT') || 'N';
          component.value = value;
          component.change(new Event('change'), element);
        },
      },
    };
    tagElement.children.unshift(switchAction);
  }

  public async gridMounted() {
    const grid = Metadata.getInstance<TekGrid>('monitoring-grid');

    try {
      grid.datasource.loading = true;
      this.monitoringModal = await ModalService.createFromJson('monitoringModal', true);

      if (!this.socket) {
        this.socket = io(Config.endPoint, {
          transports: ['websocket', 'polling'],
          query: {
            page: 'monitoring',
          },
        });
      }

      const reqParams = '/data/parametrizacoes';
      const columns = await Http.get(reqParams);
      this.addColumns(columns, grid);

      const reqMonitoring = '/data/monitoring';
      const monitoring = await Http.get(reqMonitoring);
      this.updateGridData(monitoring.data.data);
      this.addToolbarSwitchAction(grid);
      this.setLastFilter(columns, grid);
      grid.datasource.loading = false;
      this.socketListener(columns);
      this.socketEmit();
    } catch (error) {
      AlertHelper.showError(error);
    } finally {
      grid.datasource.loading = false;
    }
  }

  private setLastFilter(columns: any, grid: TekGrid) {
    const lastFilter = localStorage.getItem('LAST_FILTER_MONITORING');
    if (lastFilter) {
      const filter = JSON.parse(lastFilter);
      if (Object.keys(filter).length > 0) {
        let actualColumns = columns.data.data.filter((element: any) => element.ATIVO === 'S' && element.TIPO !== 'C')
                                                .map((element: any) => element.ID);
        // Colunas fixas do grid não podem ser removidas
        actualColumns = actualColumns.concat(['id', 'name', 'websocket', 'sat']);
        Object.keys(filter).forEach((item: any) => {
          const isNumber = !Number.isNaN(item);
          const parsedValue = isNumber ? parseInt(item, 10) : item;
          if (!actualColumns.includes(item) && !actualColumns.includes(parsedValue)) {
            delete filter[item];
          }
        });
        (grid.datasource as TekMemoryDatasource).dynamicFilter = filter;
      }
    }
  }

  private socketEmit() {
    this.socket?.emit(SocketEvent.CONNECTION);
    this.socket?.emit('login', this.socketArgs);
    this.socket?.emit('getLatestMonitoringResult');
  }

  private socketListener(columns: any) {
    this.socket?.on('monitoringResultSummary', async (response: any) => {
      const UNITS = Object.keys(response);
      const grid = Metadata.getInstance<TekGrid>('monitoring-grid');
      let updatedUnits = 0;
      (grid.datasource as TekMemoryDatasource).allData.every((unit: any) => {
        if (UNITS.includes(unit.CDFILIAL)) {
          const { QTDERROR, ...MONITOR_STATUS } = response[unit.CDFILIAL];
          Object.assign(unit, MONITOR_STATUS, { QTDERROR });
          updatedUnits += 1;
          if (UNITS.length === updatedUnits) return false;
        }
        return true;
      });
      this.updateGridData((grid.datasource as TekMemoryDatasource).allData);
    });
    this.socket?.on('login', async (response: any) => {
      const grid = Metadata.getInstance<TekGrid>('monitoring-grid');
      const { UNITS, STATUS }: any = response;
      if (UNITS.length) {
        const { allData } = grid.datasource as TekMemoryDatasource;
        const newValues: any = { websocket: STATUS };
        if (STATUS === 'N') {
          newValues.QTDERROR = {};
        }
        DataSourceHelper.updateGridRow(grid, 'CDFILIAL', UNITS, newValues, allData);
        this.updateGridData((grid.datasource as TekMemoryDatasource).allData);
      }
    });

    this.socket?.on('disconnect', () => {
      AlertHelper.showError('A página será recarregada para restabelecer a conexão.');
      setTimeout(() => {
        document.location.reload();
      }, 5000);
    });
    columns.data.data.forEach((element: any) => {
      if (element.ATIVO === 'S' && element.TIPO !== 'C') {
        this.socket?.on(element.ID, async (response: any) => {
          const grid = Metadata.getInstance<TekGrid>('monitoring-grid');
          const unit: any = (grid.datasource as TekMemoryDatasource).allData.find(({ CDFILIAL }: any) => CDFILIAL === response.CDFILIAL);

          if (!unit) return;
          unit.QTDERROR = unit.QTDERROR || {};
          unit.QTDERROR[response.ID] = response.QTDERROR.toString();
          unit[response.ID] = response.STATUS;
          if (element.TIPO === 'I' && element.NOME === 'SAT') {
            unit.sat = response.STATUS === 'S';
          }
          this.updateGridData((grid.datasource as TekMemoryDatasource).allData);
          grid.reload();
        });
      }
    });
  }

  private addColumns(columns: any, grid: TekGrid) {
    this.assConfig = columns.data.data;
    columns.data.data.forEach((element: any) => {
      if (element.ATIVO === 'S' && element.TIPO !== 'C') {
        this.columnsId.push(element.ID.toString());
        const newColumn = this.getNewColumn(element, grid);
        if (element.NOME === 'SAT') {
          this.satColumnId = element.ID.toString();
        } else {
          newColumn.filterProps = this.getNewColumnFilterProps(element);
        }
        grid.columns.push(newColumn);
      }
    });
  }

  private getNewColumn(element: any, grid: TekGrid) {
    const newColumn: ITekGridColumn = {
      name: element.ID.toString(),
      label: element.NOME,
      type: 'action',
      maxWidth: '100px',
      children: [
        {
          name: `button_${element.ID}`,
          component: 'ZdButton',
          cssClass: 'monitoring-btn',
          small: true,
          icon: true,
          iconName: 'mdi-brightness-1',
          conditions: {
            color: this.getButtonColor.bind(this),
            label: this.getButtonLabel.bind(this),
          },
          events: {
            click: this.buttonClickMonitoring.bind(this),
          },
        },
      ],
    };
    return new TekGridColumn(newColumn, grid);
  }

  private getButtonLabel({ column, row }: any) {
    const { QTDERROR = {} } = row;
    if (column.name in QTDERROR && row.websocket === 'S') {
      return QTDERROR[column.name];
    }
    return '';
  }

  private getNewColumnFilterProps(element: any) {
    const filterProps: IFilterPropsComponent = {
      name: element.ID.toString(),
      label: element.NOME,
      relation: 'AND',
      operation: 'EQUALS',
      grid: {
        sm: 3,
      },
      component: 'ZdSelect',
      autocomplete: false,
      dataText: ['text'],
      dataValue: 'value',
      datasource: {
        data: [
            {
              value: 'S',
              text: 'Sucesso',
            },
            {
              value: 'N',
              text: 'Erro',
            },
        ],
      },
    };
    return filterProps;
  }

  public closeModal() {
    this.monitoringModal.hide();
  }

  public modalGridMounted() {
    const modalGrid = Metadata.getInstance<TekGrid>('monitoring-modal-grid');
    modalGrid.datasource.uniqueKey = 'NAME';
    const column = <TekGridColumn> modalGrid.getColumn('NAME');
    modalGrid.datasource.loading = true;
    this.clickedAssConfig.CDFILIAL = this.cdfilial;

    this.socket?.emit('detailMonitoring', this.clickedAssConfig);
    this.socket?.on('detailMonitoringResult', async (response: any) => {
      this.isVisibleModalColumn = false;
      if (response && response.rows && response.rows.length) {
        if (this.clickedAssConfig.NOME === 'SAT') {
          this.isVisibleModalColumn = true;
        } else if (response.metaData) {
          response.metaData.forEach((element: any) => {
            const newColumn = new TekGridColumn({ ...column }, modalGrid);
            newColumn.name = element.name;
            newColumn.label = element.name;
            newColumn.isVisible = true;
            modalGrid.columns.push(newColumn);
          });
        } else {
          Object.keys(response.rows[0]).forEach((element: any) => {
            let newColumn;
            // S => Serviços - Loja
            // E => Apache - Loja
            // I => Interno
            const {
              TIPO,
              NOME,
            } = this.clickedAssConfig;
            let buildStatusButton = ['S', 'E'].includes(TIPO);
            buildStatusButton = buildStatusButton || (TIPO === 'I' && NOME === 'Serviços Updater');
            if (buildStatusButton && element === 'STATUS') {
              newColumn = this.getServicesStatusColumn(element, modalGrid);
              modalGrid.datasource.uniqueKey = 'DESCRICAO';
            } else {
              newColumn = new TekGridColumn({ ...column }, modalGrid);
              newColumn.name = element;
              newColumn.label = element;
              newColumn.isVisible = true;
            }
            modalGrid.columns.push(newColumn);
          });
        }
        modalGrid.datasource.updateData(response.rows);
      }
      modalGrid.datasource.loading = false;
    });
  }

  private getServicesStatusColumn(element: any, grid: TekGrid) {
    const newColumn: ITekGridColumn = {
      name: element,
      label: element,
      type: 'action',
      maxWidth: '100px',
      fixed: true,
      children: [
        {
          name: `button_${element}`,
          component: 'ZdButton',
          small: true,
          icon: true,
          iconName: 'mdi-brightness-1',
          conditions: {
            color: this.getServicesButtonColor.bind(this),
          },
        },
      ],
    };
    return new TekGridColumn(newColumn, grid);
  }

  public gridBeforeDestroy() {
    this.monitoringModal?.destroy();
    this.socket?.removeAllListeners();
    this.socket?.disconnect();
    ModalService.getModal('monitoring-grid-filter-modal')?.destroy();
  }

  public getButtonColor({ column, row }: IDictionary<any>) {
    let cor;
    const isActiveConn = row.websocket === 'S';
    if (column.name === 'websocket') {
      cor = isActiveConn ? 'green' : 'red';
    } else if (this.satColumnId && column.name === this.satColumnId && row.sat === 'N') {
      cor = 'gray';
    } else if (column.label === 'Serviços Updater') {
      if (!isActiveConn) return 'red';
      if (!row[column.name]) return 'gray';
      cor = row[column.name] === 'S' ? 'green' : 'red';
    } else {
      cor = row[column.name] === 'S' && isActiveConn ? 'green' : 'red';
    }
    return cor;
  }

  public getSatButtonColor({ row }: IDictionary<any>) {
    if (row.sat === 'N') return 'gray';
    return row.sat === 'S' ? 'green' : 'red';
  }

  public getServicesButtonColor({ row }: IDictionary<any>) {
    return row.STATUS === 'S' ? 'green' : 'red';
  }

  public buttonClickMonitoring(monitor:any) {
    this.monitoringModal.destroy();
    if (monitor.component.color === 'green') {
      AlertHelper.showSuccess('Tudo certo com esse monitoramento!');
    } else if (monitor.column.name === this.satColumnId && monitor.row.sat === 'N') {
      AlertHelper.showAlert('Essa filial não utiliza modelo fiscal SAT');
    } else if (monitor.row.websocket === 'S') {
      if (monitor.column.label === 'Serviços Updater' && monitor.component.color === 'gray') {
        return AlertHelper.showAlert('Essa filial não possui serviços gerenciados pelo Assistant');
      }
      this.cdfilial = monitor.row.CDFILIAL;
      this.clickedAssConfig = this.assConfig.find((element: any) => element.ID.toString() === monitor.column.name);
      const { CDFILIAL = '', NMFILIAL = '' } = monitor.row;
      ModalService.createFromJson('monitoringModal', true).then((modal: any) => {
        this.monitoringModal = modal;
        const header = this.monitoringModal.children.find((child:any) => child.name === 'monitoring-modal-header');
        const grid = this.monitoringModal.children.find((child:any) => child.name === 'monitoring-modal-grid');
        if (grid) {
          grid.title = this.clickedAssConfig.DESCRICAO;
        }
        if (header) {
          const unitDescription = header.leftSlot.find((item: any) => item.name === 'unit-description');
          if (unitDescription) {
            unitDescription.text = `Filial: ${CDFILIAL} - ${NMFILIAL}`;
          }
        }
        this.monitoringModal.show();
      });
    } else {
      AlertHelper.showAlert('A conexão deve estar ativa para visualizar os monitoramentos');
    }
  }

  private updateGridData(data: any) {
    const grid = Metadata.getInstance<TekGrid>('monitoring-grid');
    data.sort(this.orderGridData.bind(this));
    (grid.datasource as TekMemoryDatasource).allData = data;
    grid.datasource.data = data;
    grid.datasource.get();
  }

  private orderGridData(item1: any, item2: any) {
    if (item1.websocket === 'N' && item2.websocket === 'S') {
      return -1;
    }
    if (item1.websocket === 'S' && item2.websocket === 'N') {
      return 1;
    }

    let errorsItem1 = item1.sat === 'E' ? 1 : 0;
    let errorsItem2 = item2.sat === 'E' ? 1 : 0;

    this.columnsId.forEach((columnId:any) => {
      if (item1[columnId] === 'N') errorsItem1 += 1;
      if (item2[columnId] === 'N') errorsItem2 += 1;
    });

    if (errorsItem1 > errorsItem2) return -1;
    if (errorsItem1 < errorsItem2) return 1;
    if (item1.CDFILIAL < item2.CDFILIAL) return -1;
    return 1;
  }
}
