import Prism from 'prismjs';
import DataSourceHelper from '@/helpers/DataSourceHelper';
import AlertHelper from '@/helpers/AlertHelper';
import { ErrorHandler } from '@/helpers/ErrorHandler';
import { Http, IEventParam, Metadata, Config } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import {
  DialogService, Form, LoadingService, Modal,
  ModalService, Select, Text, TextInput, IterableComponentRender,
  ITabsBeforeChangeEventParam, Tab, Radio,
} from '@zeedhi/common';
import { Highlight } from '@zeedhi/zd-highlight-common';
import { io, Socket } from 'socket.io-client';
import { SocketEvent } from '@/socket/socket-event';
import { ConfigIni } from '../interfaces/configIni';
import 'prismjs/components/prism-markup-templating.min';
import 'prismjs/components/prism-sql.min';

export class ParametrizacoesController {
  private paramEditModal!: Modal;

  private paramFilialEditModal!: Modal;

  private socketArgs = {
    email: 'assistant@teknisa.com',
    password: 'nineninenine',
    slave: false,
  };

  public isVisibleRowHighlight = true;

  public socket?: Socket;

  private dataSourceEdit = [{
      value: 'I',
      text: 'Assistant (Proprietário)',
  }, {
      value: 'A',
      text: 'Base de Dados - Cloud',
  }, {
      value: 'L',
      text: 'Base de Dados - Loja',
  }, {
      value: 'C',
      text: 'Apache - Cloud',
  }, {
      value: 'E',
      text: 'Apache - Loja',
  }, {
      value: 'S',
      text: 'Serviços - Loja',
  }];

  private dataSourceInsert = [{
      value: 'A',
      text: 'Base de Dados - Cloud',
  }, {
      value: 'L',
      text: 'Base de Dados - Loja',
  }, {
      value: 'C',
      text: 'Apache - Cloud',
  }, {
      value: 'E',
      text: 'Apache - Loja',
  }, {
      value: 'S',
      text: 'Serviços - Loja',
  }];

  public beforeChange({ tabIndex, component }: ITabsBeforeChangeEventParam) {
    const currentTab: Tab = component.tabs[tabIndex];
    if (currentTab && currentTab.name === 'tabParamUnd') {
      this.socket?.emit(SocketEvent.CONNECTION);
      this.socket?.emit('login', this.socketArgs);
    }
  }

  public async onCreatedTabs() {
    this.paramEditModal = await ModalService.createFromJson('parametrizacoes-edit', true);
    this.socketListener();
  }

  private getMainGrid(): TekGrid {
    const grid = Metadata.getInstance<TekGrid>('gridParams');
    return grid;
  }

  private getFilialGrid(): TekGrid {
    const grid = Metadata.getInstance<TekGrid>('gridParamFilial');
    return grid;
  }

  private createSocketConnection() {
    if (!this.socket) {
      this.socket = io(Config.endPoint, {
        transports: ['websocket', 'polling'],
        query: { page: 'parametrizacoes' },
      });
    }
  }

  public async mountedParamGeral() {
    const grid = this.getMainGrid();

    try {
      grid.datasource.loading = true;
      const param = await Http.get('/data/parametrizacoes');

      grid.datasource.updateData(param.data.data);
    } catch (error) {
      AlertHelper.showError(error);
    } finally {
      grid.datasource.loading = false;
    }
  }

  public async mountedParamFilial() {
    const grid = this.getFilialGrid();

    try {
      grid.datasource.loading = true;

      const paramFilial = await Http.get('/data/parametrizacoesFilial');

      grid.datasource.updateData(paramFilial.data.data);
      grid.datasource.loading = false;
    } catch (error) {
      AlertHelper.showError(error);
    } finally {
      grid.datasource.loading = false;
    }
  }

  private getParamFilialGrid(): TekGrid | null {
    try {
    const gridParamFilial = Metadata.getInstance<TekGrid>('gridParamFilial');
    return gridParamFilial;
    } catch (error) {
      return null;
    }
  }

  private socketListener() {
    this.createSocketConnection();
    this.socket?.on('updaterConnected', async ({ UNITS, CONNECTED, WARNING }) => {
      if (WARNING) return;
      const grid = this.getParamFilialGrid();
      if (!grid) {
        return;
      }
      const UNIT_CODES = UNITS.map((unit:any) => unit.CDFILIAL);
      const { allData } = (<any>grid.datasource);
      const updatedRows = DataSourceHelper.updateGridRow(grid, 'CDFILIAL', UNIT_CODES, { CONNECTION_STATUS: CONNECTED }, allData);
      const connectionColumn = grid.getColumn('CONNECTION_STATUS');
      updatedRows?.forEach((row: any) => {
        grid.reapplyConditions(row, [connectionColumn]);
      });
    });

    this.socket?.on('updateConfigIniFinish', async (result) => {
      try {
        if (result.error) {
          ErrorHandler.getMessage(result);
        }

        const { params } = result;
        const requestParams = {
          CDOPERADOR: localStorage.getItem('CDOPERADOR'),
          CDFILIAL: this.getUnitInParametersModal(),
          CONFIG: params,
        };

        const { data } = await Http.post('/updater/saveConfigIniByUnit', requestParams);

        if (data.error) {
          ErrorHandler.getMessage(data);
        }

        AlertHelper.showSuccess(result.message);
      } catch (error) {
        AlertHelper.showError(error);
      } finally {
        this.closeParamFilialEditModal();
        LoadingService.hide();
      }
    });

    this.socket?.on('testConnectionFinish', (params) => {
      try {
        if (params.error) {
          ErrorHandler.getMessage({ message: params.message, params: { ERROR: params.detailError } });
        }

        AlertHelper.showSuccess(params.message);
      } catch (error) {
        AlertHelper.showError(error);
      } finally {
        LoadingService.hide();
      }
    });
  }

  public getConnectionColor({ row }: any) {
    const color = row.CONNECTION_STATUS ? 'green' : 'red';
    return row.WARNING ? '#FFCD01' : color;
  }

  public moreInfoConnection({ row }: any) {
    if (!row.WARNING) return;
    if (row.CONNECTION_STATUS) {
      AlertHelper.showAlert('DUPLICATE_CONNECTION_ATTEMPT');
      this.socket?.emit('detailStatusConnection', row.CDFILIAL);
    } else {
      AlertHelper.showAlert('INCOMPATIBLE_ASSISTANT_VERSION');
    }
  }

	public gridBeforeDestroy() {
		this.paramEditModal?.destroy();
    this.paramFilialEditModal?.destroy();
    this.socket?.removeAllListeners();
    this.socket?.disconnect();
	}

  public loadLanguage() {
    const { code, language } = Metadata.getInstance<Highlight>('COMANDO');
    return Prism.highlight(code || '', Prism.languages[language], language);
  }

	public async gridCellClick({ component, row } : any) {
		if (!component.editing) {
			await this.paramEditModal.show();
			const form = Metadata.getInstance<Form>('edit-modal-form');
      const typeColumn: Select = form.getChildInstance('TIPO');
      const nameColumn: TextInput = form.getChildInstance('NOME');
      const commandObsColumn: Text = form.getChildInstance('COMMAND_OBS');
      form.value = row;
      if (row.TIPO === 'I') {
        this.isVisibleRowHighlight = false;
        typeColumn.datasource.updateData(this.dataSourceEdit);
        typeColumn.readonly = true;
        nameColumn.readonly = true;
        commandObsColumn.text = 'Comandos internos não podem ser alterados';
      } else {
        typeColumn.datasource.updateData(this.dataSourceInsert);
        typeColumn.readonly = false;
        nameColumn.readonly = false;
        this.changeModalFormFields(row.TIPO);
      }
		}
	}

  public capitalizeFirstLetter(string: String) {
    return string.charAt(0).toUpperCase() + string.slice(1);
  }

  public async buttonConfigClick({ row } : any) {
    try {
      if (!row.CONNECTION_STATUS) {
        ErrorHandler.getMessage({ message: 'CONNECTION_ACTIVE_REQUIRED_CHANGE_CONFIG' });
      }

      LoadingService.show();

      this.paramFilialEditModal?.destroy();
      this.paramFilialEditModal = await ModalService.createFromJson('parametrizacoes-filial-configini', true);
      await this.paramFilialEditModal.show();

      const params = { CDFILIAL: row.CDFILIAL };
      const modal = this.paramFilialEditModal;
      const setCdfilial: any = modal.getChildInstance('CDFILIAL');
      setCdfilial.text = row.CDFILIAL;

      const { data } = await Http.post('/updater/getConfigIniByUnit', params);

      if (data.error) {
        ErrorHandler.getMessage(data);
      }

      const configInterface : ConfigIni = data.data;
      const card: any = modal.getChildInstance('collapseCardIterable');
      const titleHeader = Metadata.getInstance<Text>('view-modal-title-filial');
      titleHeader.text += ` Filial ${row.CDFILIAL} - ${row.NMFILIAL}`;

      if (Object.keys(configInterface).length > 0) {
        Object.entries(configInterface).forEach(([group, fields]) => {
          const obj: any = {
            id: `collapseCard-${group}`,
            name: this.capitalizeFirstLetter(group),
            list: [],
          };

          Object.entries(fields).forEach(([prop, valor]) => {
            obj.list.push({
              id: `field-${group}-${prop}`,
              name: prop,
              value: valor,
              label: prop,
              readonly: this.getReadOnlyProp(group, prop),
            });
          });

          card.datasource.data.push(obj);
        });
      }
    } catch (error) {
      AlertHelper.showError(error);
    } finally {
      LoadingService.hide();
    }
  }

  private getReadOnlyProp(group: string, prop: string) {
    if (group === 'database' || (group === 'environment' && prop === 'CDFILIAL')) {
      return false;
    }

    return true;
  }

  public modalConfig({ component } : IEventParam<IterableComponentRender>) {
    const parentIterable = Metadata.getInstance<IterableComponentRender>('collapseCardIterable');
    const id = component.name.split('_').pop();
    const parentIndex = parentIterable.datasource.data.findIndex((row) => row.id === id);
    component.datasource.data = parentIterable.datasource.data[parentIndex].list;
  }

  public closeParamFilialEditModal() {
		this.paramFilialEditModal.hide();
    this.paramFilialEditModal.destroy();
	}

  private getUnitInParametersModal() {
    const cdFilial: any = this.paramFilialEditModal.getChildInstance('CDFILIAL');
    if (!cdFilial) return null;
    return cdFilial.text;
  }

  public async saveConfigIni() {
    try {
      const card: any = this.paramFilialEditModal.getChildInstance('collapseCardIterable');
      const dados: any = {};

      if (card.datasource.data.length === 0) {
        ErrorHandler.getMessage({ message: 'NO_DATA_TO_SAVE' });
      }

      LoadingService.show();

      card.datasource.data.forEach((group:any) => {
        const form: any = Metadata.getInstance<Form>(`formCard-${group.name}`);

        if (!form.validate()) {
          ErrorHandler.getMessage({ message: 'FAIL_FORM_VALIDATION' });
        }

        const inputs = [...form.element].filter((item) => item.type === 'text');
        const obj: any = {};

        inputs.forEach((row: any) => {
          const key = row.name.split('-').pop();
          obj[key] = row.value;
        });

        dados[group.name.toLowerCase()] = obj;
      });

      const params = {
        CDFILIAL: this.getUnitInParametersModal(),
        CONFIG: dados,
      };

      this.socket?.emit('updateConfigInit', params);
    } catch (error) {
      AlertHelper.showError(error);

      this.closeParamFilialEditModal();
      LoadingService.hide();
    }
  }

  public async buttonHistoryClick({ row }: any) {
    try {
      LoadingService.show();

      this.paramFilialEditModal?.destroy();
      this.paramFilialEditModal = await ModalService.createFromJson('parametrizacoes-filial-history', true);

      const params = { CDFILIAL: row.CDFILIAL };

      const { data } = await Http.post('/updater/getConfigHistoryByUnit', params);

      if (data.error) {
        ErrorHandler.getMessage(data.message);
      }

      const grid = this.paramFilialEditModal.children[1];
      grid.datasource.data = data.data;
      await this.paramFilialEditModal.show();
      const titleHeader = Metadata.getInstance<Text>('history-modal-title-filial');
      titleHeader.text += ` Filial ${row.CDFILIAL} - ${row.NMFILIAL}`;
    } catch (error) {
      AlertHelper.showError(error);
    } finally {
      LoadingService.hide();
    }
  }

  public buttonTestConnectionClick({ row }: any) {
    try {
      if (!row.CONNECTION_STATUS) {
        ErrorHandler.getMessage({ message: 'CONNECTION_ACTIVE_REQUIRED_TEST_DB' });
      }

      LoadingService.show();
      this.socket?.emit('testConnection', row.CDFILIAL);
    } catch (error) {
      AlertHelper.showError(error);
      LoadingService.hide();
    }
  }

  public changeModalTipo(params: any) {
    this.changeModalFormFields(params.value);
  }

  private changeModalFormFields(type: string) {
    const form = Metadata.getInstance<Form>('edit-modal-form');

    const commandApacheColumn: TextInput = form.getChildInstance('COMANDO_APACHE');
    const commandServicesColumn: TextInput = form.getChildInstance('COMANDO_SERVICOS');
    const commandObsColumn: Text = form.getChildInstance('COMMAND_OBS');
    const commandComponent = Metadata.getInstance<Highlight>('COMANDO');
    const commandObsRadioButton: Radio = form.getChildInstance('COMPLEMENTOTIPO');

    commandApacheColumn.removeValidation('required');
    commandServicesColumn.removeValidation('required');

    switch (type) {
      case 'A':
      case 'L':
        commandObsColumn.text = 'Esse monitoramento será considerado como sucesso quando:';
        this.isVisibleRowHighlight = true;
        commandServicesColumn.isVisible = false;
        commandApacheColumn.isVisible = false;
        commandComponent.code = form.value.COMANDO;
        commandObsRadioButton.isVisible = true;
        break;

      case 'C':
      case 'E':
        this.isVisibleRowHighlight = false;
        commandServicesColumn.isVisible = false;
        commandApacheColumn.isVisible = true;
        commandApacheColumn.addValidation('required', {});
        form.value.COMANDO_APACHE = form.value.COMANDO;
        commandObsColumn.text = 'Esse monitoramento será considerado como sucesso quando a requisição GET feita para os endereços informados funcionar.';
        commandObsRadioButton.isVisible = false;
        break;

      case 'S':
        this.isVisibleRowHighlight = false;
        commandApacheColumn.isVisible = false;
        commandServicesColumn.isVisible = true;
        commandServicesColumn.addValidation('required', {});
        form.value.COMANDO_SERVICOS = form.value.COMANDO;
        commandObsColumn.text = 'Esse monitoramento será considerado como sucesso quando todos os serviços informados estiverem em execução.';
        commandObsRadioButton.isVisible = false;
        break;

      case 'I':
        this.isVisibleRowHighlight = true;
        commandServicesColumn.isVisible = false;
        commandApacheColumn.isVisible = false;
        commandComponent.code = form.value.COMANDO;
        commandObsColumn.text = 'Esse monitoramento será considerado como sucesso quando a consulta não retornar nenhum registro.';
        commandObsRadioButton.isVisible = false;
        break;

      default:
    }
  }

	public closeParamEditModal() {
		this.paramEditModal.hide();
	}

  public async addClick() {
    await this.paramEditModal.show();
    const form = Metadata.getInstance<Form>('edit-modal-form');
    const typeColumn: Select = form.getChildInstance('TIPO');
    typeColumn.datasource.updateData(this.dataSourceInsert);
    typeColumn.readonly = false;
    form.value.TIPO = 'A';
  }

  public getConditionalCssClass(param: any) {
    return param.row.ATIVO === 'N' ? 'ass-inactive' : '';
  }

  public async updateInModal() {
    try {
      LoadingService.show();

      const grid = Metadata.getInstance<TekGrid>('gridParams');
      const form = Metadata.getInstance<Form>('edit-modal-form');
      const commandComponent = Metadata.getInstance<Highlight>('COMANDO');

      if (!form.validate()) {
        ErrorHandler.getMessage({ message: 'FAIL_FORM_VALIDATION' });
      }

      if ((form.value.TIPO === 'A' || form.value.TIPO === 'L') && !form.value.COMPLEMENTOTIPO) {
        ErrorHandler.getMessage({ message: 'REQUIRED_RECORD_TYPE' });
      }

      if (!commandComponent.code && (form.value.TIPO === 'A' || form.value.TIPO === 'L')) {
        ErrorHandler.getMessage({ message: 'REQUIRED_COMMAND' });
      }

      switch (form.value.TIPO) {
        case 'I':
          form.value.COMANDO = 'INTERNO';
          break;

        case 'A':
        case 'L':
          form.value.COMANDO = commandComponent.code;
          break;

        case 'C':
        case 'E':
          form.value.COMANDO = form.value.COMANDO_APACHE;
          break;

        default:
          form.value.COMANDO = form.value.COMANDO_SERVICOS;
      }
      const uri = '/data/monitoringConfig';
      let response = null;
      if (form.value.ID) {
        response = await Http.put(`${uri}/${form.value.ID}`, form.value);
      } else {
        response = await Http.post(uri, form.value);
      }
      const { data } = response;

      if (data.error) {
        ErrorHandler.getMessage(data);
      }

      this.paramEditModal.hide();
      grid.datasource.get();

      AlertHelper.showSuccess(data.message);
    } catch (error) {
      AlertHelper.showError(error);
    } finally {
      LoadingService.hide();
    }
  }

  public async beforeDelete(params: any) {
    params.event.preventDefault();
    const grid = Metadata.getInstance<TekGrid>('gridParams');

    DialogService.show({
			name: `confirm_${grid.name}`,
			type: 'warning',
			title: 'Exclusão',
			text: 'Deseja realmente excluir os registros selecionados?',
			buttons: [
				{
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          outline: true,
          events: {
            click: () => DialogService.hide(),
          },
        },
				{
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: this.deleteConfig,
        },
			],
		});
  }

  private deleteConfig: any = {
    click: () => this.doBeforeDelete(),
  };

  private async doBeforeDelete() {
    try {
      const grid = Metadata.getInstance<TekGrid>('gridParams');

      const ids: any = [];
      grid.selectedRows.forEach((element) => {
        if (element.ID) {
          ids.push(element.ID);
        }
      });

      LoadingService.show();

      const { data } = await Http.delete(`/data/monitoringConfig/${ids.join(',')}`);

      if (data.error) {
        ErrorHandler.getMessage(data);
      }

      this.paramEditModal.hide();
      grid.datasource.get();
      grid.selectedRows = [];

      AlertHelper.showSuccess(data.message);
    } catch (error) {
      AlertHelper.showError(error);
    } finally {
      LoadingService.hide();
      DialogService.hide();
    }
  }
}
