import AlertHelper from '@/helpers/AlertHelper';
import { LoadingService } from '@zeedhi/common';
import { Metadata, Http } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class VersionController {
    public getIsVisibleApprove({ row }: any) {
        return row.IDAPROVVER === 'S';
    }

    public getIsVisibleRemove({ row }: any) {
        return row.IDAPROVVER === 'N';
    }

    public approveVersion({ row }: any) {
        const grid = Metadata.getInstance<TekGrid>('version-grid');

        const isApprove = row.IDAPROVVER === 'N';
        const params = {
            ID: row.ID,
            IDAPROVVER: isApprove ? 'S' : 'N',
        };

        LoadingService.show();
        Http.put('updateUpdaterProdVersaoAprov', params).then(() => {
            LoadingService.hide();

            grid.datasource.data.map((datasource) => {
                if (datasource.ID === params.ID) {
                    datasource.IDAPROVVER = params.IDAPROVVER;
                }

                return true;
            });

            grid.datasource.get();

            const message = isApprove ? 'Versão aprovada com sucesso!' : 'Aprovação removida com sucesso!';
            AlertHelper.showSuccess(message);
        }).catch((error) => {
            LoadingService.hide();
            AlertHelper.showError(error);
        });
    }

    public async gridMounted() {
        const grid = Metadata.getInstance<TekGrid>('version-grid');

        try {
          grid.datasource.loading = true;

          const versionConfig = await Http.get('/updaterProdutosVersaoConfig');

          grid.datasource.updateData(versionConfig.data.data);
        } catch (error) {
          AlertHelper.showError(error);
        } finally {
          grid.datasource.loading = false;
        }
      }
}
