import AlertHelper from '@/helpers/AlertHelper';
import { Modal, ModalService } from '@zeedhi/common';

class WarningController {
  private modal!: Modal;

  public async create(modal: string, events: object | any): Promise<void> {
    this.modal = await ModalService.createFromJson(modal, true);
    const closeModal = this.getCloseButton();
    const statusColumn = this.getColumn('STATUS_CONNECTION');
    if (closeModal) {
      closeModal.events = {
        click: () => {
          if (events && typeof events.close === 'function') {
            events.close.apply(this, []);
          }
          this.close();
        },
      };
    }
    if (statusColumn) {
      const btnConnection = statusColumn.children.find((child: any) => child.name === 'btnConnection');
      if (btnConnection) {
        btnConnection.conditions = {
          color: this.getConnectionColor.bind(this),
        };
      }
    }
  }

  public async openUpdaterDuplicateConnModal(INFO:any) {
    const { ACTIVE, ...hosts } = INFO;
    const dataset = Object.keys(hosts).reduce((acc: any, hostname: any, index: number) => acc.concat({
        ID: index,
        CONNECTION_STATUS: false,
        hostname,
        address: hosts[hostname],
    }), []);
    dataset.unshift({ ...ACTIVE, CONNECTION_STATUS: true, ID: dataset.length });
    const gridDuplicateConn = this.modal?.children.find((el) => el.name === 'duplicate-conn-grid');
    if (gridDuplicateConn instanceof Object) {
      gridDuplicateConn.datasource.data = dataset;
      this.modal?.show();
    }
  }

  private getConnectionColor({ row }: any) {
      const color = row.CONNECTION_STATUS ? 'green' : 'red';
      return row.WARNING ? '#FFCD01' : color;
  }

  private getGrid() {
    return this.getChildElement('duplicate-conn-grid');
  }

  private getColumn(name: string) {
    const grid = this.getGrid();
    if (grid) {
      return grid.columns.find((column: any) => column.name === name) || null;
    }
    return null;
  }

  private close() {
    this.modal?.hide();
  }

  public destroyModal(modal: string): void {
    ModalService.getModal(modal)?.destroy();
  }

  private getHeader() {
    const header = this.getChildElement('duplicate-conn-header');
    return header;
  }

  private getCloseButton() {
    const header = this.getHeader();
    if (!header) return null;
    const closeButton = header.rightSlot.find((child: any) => child.name === 'closeModalButton');
    return closeButton;
  }

  private getChildElement(name: any) {
    const element = this.modal?.children.find((child: any) => child.name === name);
    return element || null;
  }

  public moreInfoConnection(row: any, socket: any) {
    if (!row.WARNING) return;
    if (row.CONNECTION_STATUS) {
      AlertHelper.showAlert('Foi realizada uma tentativa de conexão da mesma unidade em outra instalação.');
      if (socket && typeof socket.emit === 'function') {
        const { CDFILIAL, CDCAIXA = '' } = row;
        socket?.emit('detailStatusConnection', { CDFILIAL, CDCAIXA });
      }
    } else {
      AlertHelper.showAlert('A versão atual do Assistant Updater não é compatível.');
    }
  }
}

export default new WarningController();
