import { SocketEvent } from '@/socket/socket-event';
import { Config, Http, IDictionary, IEventParam, Loader } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { io, Socket } from 'socket.io-client';
import { AlertService, LoadingService, Modal, ModalService } from '@zeedhi/common';
import Router from '../router';

export class WatchController {
  public socket?: Socket;

  private watchModal!: Modal;

  private cdfilial: any;

  private columnSelected: any;

  /** Style Status do Sat */
  public getConditionalStyle({ row }: IDictionary<any>) {
    return row.sat ? 'green' : 'red';
  }

  public getConditionalColumnStyle({ column, row }: IDictionary<any>) {
    let cor;
    const situacao = 'situacao';

    [row].forEach((element:any) => {
      if (column.name === 'websocket') {
        cor = (element[situacao] === 'Ativo') ? 'green' : 'red';
      } else if (column.name === 'sat' && element[column.name] === 'NOT_SAT') {
        cor = 'gray';
      } else {
        cor = 'red';
        if (element[situacao] === 'Ativo') {
          cor = this.switchColor(element[column.name]);
        }
      }
    });
    return cor;
  }

  /** Menssagem Status do Sat */
  public getConditionalMessage({ row }: IDictionary<any>) {
    if (
      (row.sat === '') && (row.socketId !== '')
    ) {
      return 'Erro ao tentar uma comunicação com o SAT.';
    }
    return row.mensagem;
  }

  public startModal() {
    Loader.getInstance('SatController').createFullscreenModal();
    Loader.getInstance('SalesController').createFullscreenModal();
    Loader.getInstance('DeliveryController').createFullscreenModal();
    Loader.getInstance('ParametersController').createFullscreenModal();
    Loader.getInstance('SefazController').createFullscreenModal();
  }

  public async createFullscreenModal() {
    // const watchModalName:any = router.currentRoute.query.monitoring;
    // this.fullscreenModal = await ModalService.createFromJson(watchModalName, true);
  }

  /**
   * Método executado quando o grid é criado
   */
  public gridMounted({ component }: IEventParam<TekGrid>) {
    // inicializa o socket
    if (Router.currentRoute.params.name === 'watch') {
      Router.push('watch/monitoring');
    }
    const rq = 'connect?obj=';
    const conexao = Config.metadataEndPoint + rq + (this.columnSelected || Router.currentRoute.params.name);
    const ioOpt = {
      transports: ['websocket', 'polling'],
    };

    const req = Http.get(conexao);
    component.datasource.loading = true;
    req.then((response) => {
      response.data.conexao.forEach((element:any) => {
        // Garante que o socket faça uma única conexão
        if (!this.socket) {
          this.socket = io(Config.endPoint, ioOpt);
        }

        /** Emit */
        element.args = element.args || {};
        element.args.CDFILIAL = this.cdfilial;
        // Se for Login, crie o CONNECT, se não, só emita.
        if (element.route === 'login') {
          console.log('Emitindo: ', element.route);
          this.socket.emit(SocketEvent.CONNECTION)
            .emit(element.route, element.args);
            console.log('ID: ', this.socket.id);
        } else {
          this.socket
            .emit(element.route, element.args);
            console.log('Emitindo: ', element.route, element.args);
        }

        /** Esculta */
        this.socket
          .on(element.route, async (branc: IDictionary) => {
            /** Reconnexão, Desconectado */
            if (branc.text === 'UNIDADE CONECTADA') {
              /** Reconectou */
              component.datasource.get().then((r: IDictionary) => {
                if (r.length !== 0) {
                  branc.message.data.forEach((elements:any) => {
                    if (elements.satError) {
                      const isSatError = elements.satError.some((item:any) => !item.sat);
                      elements.sat = !isSatError;
                    }
                    component.datasource.put(elements);
                  });
                } else {
                  console.log('Componenet: ', component.datasource);
                  component.datasource.updateData(branc.message.data);
                }
              });
            } else if (branc.text === 'UNIDADE DESCONECTADA') {
              /** Disconectou */
              component.datasource.put(branc.message.data);
            }

            component.datasource.get().then((r: IDictionary) => {
              if (r.length === 0) {
                if (Router.currentRoute.params.name !== 'login'
                && element.route !== 'login'
                && branc.message.data.length > 0
                && Object.keys(branc.message.data[0]).length > 2) {
                  component.datasource.updateData(branc.message.data);
                  component.datasource.loading = false;
                } else if (element.route === 'login'
                  && (Router.currentRoute.params.name === 'monitoring' && !this.columnSelected)) {
                    component.datasource.updateData(branc.message);
                    component.datasource.loading = false;
                }
              } else if (
                branc.message.data.length > 0
              ) {
                if (
                  element.route === 'sefaz'
                  && Object.prototype.toString.call(branc.message.data) === '[object Array]'
                  && branc.message.data.length > 1
                  && branc.message.data[0]
                  && branc.message.data[0].idnormal !== undefined
                ) {
                  const coponetAtual = r.filter((v: IDictionary) => {
                    branc.message.data.forEach((elements:any) => {
                      if (elements.id === v.id) {
                        elements.sefaz = this.seFazStatus(elements);
                        Object.assign(v, elements);
                        return v;
                      }
                      return false;
                    });
                    return v;
                  });
                  if (coponetAtual[0] !== undefined) {
                    component.datasource.put(coponetAtual[0]);
                  }
                } else if (branc.message.data.length > 0) {
                  const coponetAtual = r.filter((v: IDictionary) => {
                    let ret = false;
                    if (branc.text === 'SAT') {
                      const isSatError = branc.message.data.some((item:any) => !item.sat);
                      if (branc.message.data[0].id === v.id) {
                        v.sat = !isSatError;
                        ret = true;
                      }
                    } else {
                      branc.message.data.forEach((elements:any) => {
                        if (elements.id === v.id) {
                          Object.assign(v, elements);
                          ret = true;
                        }
                      });
                    }
                    return ret;
                  });
                  if (coponetAtual[0] !== undefined) {
                    component.datasource.put(coponetAtual[0]);
                  }
                  component.datasource.loading = false;
                }
              } else {
                component.datasource.loading = false;
              }
            });
          });
      });
    }).catch((e) => {
      console.log('ERRO NA BUSCA DO MONITORING', e);
      component.datasource.loading = false;
    });

    this.createFullscreenModal();
    this.socket = io(Config.endPoint, {
      transports: ['websocket', 'polling'],
    });
    this.socket.on('disconnect', () => {
      if (Router.currentRoute.params.name === 'monitoring') {
        AlertService.show({
          name: 'buttons-alert',
          text: 'A página será recarregada para restabelecer a conexão.',
          color: 'error',
          right: true,
          top: true,
          value: false,
          buttonsSlot: [
            {
              name: 'undo-button',
              component: 'ZdButton',
              icon: true,
              iconName: 'close',
              color: 'white',
              events: {
                click: () => { AlertService.hide(); },
              },
            },
          ],
        });
        setTimeout(() => {
          document.location.reload();
        }, 5000);
      }
    });
    setInterval(() => {
      console.log('Ping: ', this.socket?.id);
      this.socket?.emit('ping', { id: this.socket?.id });
    }, 20000);
  }

  public seFazStatus(element:any): boolean {
    let cor = true;
    if (
      element.idnormal === 'CINZA'
      && element.idscan === 'CINZA'
      && element.idsvc === 'CINZA'
    ) {
      cor = true;
    } else {
      cor = false;
    }
    return cor;
  }

  private showAlert(message: string, colorStr: string) {
    AlertService.show({
      name: 'info-alert',
      text: message,
      color: colorStr,
      right: true,
      top: true,
      showDismiss: true,
      timeout: 10000,
    });
  }

  public push(monitor:any) {
    if (monitor.component.color === 'green') {
      this.showAlert('Tudo certo com esse monitoramento!', 'success');
    } else if (monitor.column.name === 'sat' && monitor.row.sat === 'NOT_SAT') {
      this.showAlert('Essa filial não utiliza modelo fiscal SAT', '#e68129');
    } else if (monitor.row.situacao === 'Ativo') {
      this.cdfilial = monitor.row.id;
      this.columnSelected = monitor.column.name;
      LoadingService.show();
      const conexao = `${Config.metadataEndPoint}parametrizacoes`;
      Http.get(conexao).then((response) => {
        const parameters = response.data.filter((param: any) => param.nome === monitor.column.name)[0];
        const component = JSON.parse(parameters.component);
        if (component.children[1].component === 'ZdTreeGrid') {
          component.children[1].toolbarSlot[0].children[0].text += ` - ${monitor.row.name}`;
        } else {
          component.children[1].title += ` - ${monitor.row.name}`;
        }
        this.watchModal = ModalService.create(component);
        this.watchModal.show();
      }).catch((error) => {
        this.showAlert(error, 'error');
      }).finally(() => {
        LoadingService.hide();
      });
    } else {
      this.showAlert('A conexão deve estar ativa para visualizar os monitoramentos', '#e68129');
    }
  }

  public closeModal() {
    this.watchModal.hide();
  }

  /**
   * Método executado logo antes do grid ser destruído
   */
  public gridBeforeDestroy() {
    /** Fecha a conexão com o socket */
    this.watchModal?.destroy();
    console.log('Destroiu o Grid');
  }

  public gridBeforeWatchDestroy() {
    this.socket?.disconnect();
    console.log('Destroiu o Grid Watch');
  }

  /** Status do Normal */
  public getConditionalStyleNormal({ row }: IDictionary<any>) {
    if (row.situacao === 'Ativo' && row.idnormal === 'CINZA') {
      return 'gray';
    }
    if (row.situacao === 'Ativo' && row.idnormal === 'VERDE') {
      return 'green';
    }
    if (row.situacao === 'Ativo' && row.idnormal === 'AMARELO') {
      return 'yellow';
    }
    return 'red';
  }

  /** Status do Normal */
  public getConditionalNormal({ row }: IDictionary<any>) {
    let message = '';
    if (row.idnormal === 'CINZA') {
      message = 'falta Serviço ou falha de conexão.';
    } else if (row.idnormal === 'VERDE') {
      message = 'Sistema operante.';
    } else {
      message = row.message;
    }
    return message;
  }

  /** Status do Vendas */
  public getConditionalStyleScan({ row }: IDictionary<any>) {
    if (row.idscan === 'CINZA') {
      return 'green';
    }
    return 'red';
  }

  public getConditionalScan({ row }: IDictionary<any>) {
    if (row.idscan === 'CINZA') {
      return 'green';
    }
    return 'red';
  }

  /** Status do Vendas */
  public getConditionalStyleSvc({ row }: IDictionary<any>) {
    if (row.idsvc === 'CINZA') {
      return 'green';
    }
    return 'red';
  }

  public getConditionalSvc({ row }: IDictionary<any>) {
    if (row.idsvc === 'CINZA') {
      return 'green';
    }
    return 'red';
  }

  public switchColor(color:any) {
    return (color === true || color === 'Ativado') ? 'green' : 'red';
  }

  created() {
    this.socket?.disconnect();
  }

  public showConnected(component:any) {
    console.log(component);
    const { datasource } = component;
    datasource.loading = true;
    const connected: any = datasource.data.filter((item: any) => item.situacao === 'Ativo');
    datasource.updateData(connected);
    datasource.loading = false;
  }
}
