import { AlertService } from '@zeedhi/common';
import MessageHandler from '@/helpers/MessageHandler';

class Alert {
    alertColor = '#e68129';

    errorColor = 'error';

    successColor = 'success';

    tenSeconds = 10 * 1000;

    private alertMessage(name: string, text: string | Error, color:string, params?: any, timeout?: number) {
        if (text instanceof Error && text.message === '401_UNAUTHORIZED') {
            return;
        }

        let message: string;

        if (text instanceof Error) {
            message = text.message;
        } else {
            message = text;
        }

        AlertService.show({
            name,
            text: MessageHandler.getMessageByCode(message, params),
            color,
            right: true,
            top: true,
            showDismiss: true,
            timeout: timeout || this.tenSeconds,
        });
    }

    showAlert(message: string | Error, params?: any, timeout?: number, name?: string) {
        const popupName = name || 'alert-info';
        this.alertMessage(popupName, message, this.alertColor, params, timeout);
    }

    showError(message: string | any | Error, params?: any, timeout?: number, name?: string) {
        const popupName = name || 'error-info';
        this.alertMessage(popupName, message, this.errorColor, params, timeout);
    }

    showSuccess(message: string | Error, params?: any, timeout?: number, name?: string) {
        const popupName = name || 'success-info';
        this.alertMessage(popupName, message, this.successColor, params, timeout);
    }
}

export default new Alert();
