import { Config } from '@zeedhi/core';
import { io, Socket } from 'socket.io-client';
import AlertHelper from './AlertHelper';

export class SocketHelper {
  static isReconnection = false;

  static createConnection(query = {}) {
    const socket: Socket = io(Config.endPoint, {
      transports: ['websocket', 'polling'],
      query,
    });

    socket.on('connect', () => {
      if (SocketHelper.isReconnection) {
        AlertHelper.showSuccess('CONNECTED_SOCKET');
        SocketHelper.isReconnection = false;
      }
    });

    socket.on('disconnect', () => {
      AlertHelper.showError('DISCONNECTED_SOCKET');
      SocketHelper.isReconnection = true;
      socket.disconnect();
      const reconnectInterval = setInterval(() => {
        if (socket.connected) {
          clearInterval(reconnectInterval);
        } else {
          socket.connect();
        }
      }, 5000);
    });
    return socket;
  }
}
